import time
time.sleep(5)



import win32gui
import win32con
 
def minimize_all_windows():
    def _enum_windows_proc(hwnd, param):
        if win32gui.IsWindowVisible(hwnd) and win32gui.GetWindowLong(hwnd, win32con.GWL_STYLE) & win32con.WS_VISIBLE:
            win32gui.SendMessage(hwnd, win32con.WM_SYSCOMMAND, win32con.SC_MINIMIZE, 0)
 
    win32gui.EnumWindows(_enum_windows_proc, 0)
    import time
    time.sleep(2)
    import os
    os.system("cqzy")
 
minimize_all_windows()


#!/usr/bin/env python
# coding: utf-8

# 导入用于 Excel 操作的 Python 库 xlwings
import xlwings

# 导入用户自定义模块 frankyu
import frankyu

# 导入 frankyu 模块中的 excel 子模块
import frankyu.excel

# 为了简化后续代码，将 frankyu.excel 模块赋值给变量 ex
ex = frankyu.excel

# 初始化 Excel 应用程序，通过自定义模块中的方法进行初始化
app = ex.initialize_excel()

# 创建一个新的 Excel 工作簿，通过自定义模块中的方法进行创建
book = ex.create_workbook(app)

def excelpid():
    """
    获取当前运行的第一个 Excel 应用程序的进程 ID (PID)。

    思路:
    1. 遍历 xlwings 提供的所有运行中的 Excel 应用程序实例。
    2. 获取每个实例的进程 ID (PID) 并存储到列表中。
    3. 返回第一个应用程序实例的 PID。

    方法清单:
    - xlwings.apps: 获取所有运行中的 Excel 应用程序实例。
    - app.pid: 获取 Excel 应用程序实例的进程 ID (PID)。

    返回值:
        int: 当前运行的第一个 Excel 应用程序的 PID。
    """
    bbb = []  # 初始化存储所有 Excel 应用程序 PID 的列表

    for i in xlwings.apps:
        # 遍历所有运行中的 Excel 应用程序实例

        aaa = i.pid
        # 获取当前 Excel 应用程序的 PID

        bbb.append(aaa)
        # 将当前 Excel 应用程序的 PID 添加到列表中

    app = xlwings.apps[bbb[0]]
    # 获取列表中第一个 Excel 应用程序实例，通过其 PID

    return app.pid
    # 返回第一个 Excel 应用程序实例的 PID

# 调用 excelpid 函数
excelpid()


# 获取当前 Excel 应用程序的 PID
eee = excelpid()

# 获取第一个工作薄的第一个工作表中的单元格 A1
rng = xlwings.apps[eee].books[0].sheets[0].range("A1")

# 将公式 "=2+3" 设置为单元格 A1 的值
rng.value = "=2+3"

# 打印所有运行中的 Excel 应用程序实例
xlwings.apps

# 遍历并打印所有运行中的 Excel 实例及其打开的工作簿名称
for i in xlwings.apps:
    # 遍历所有运行中的 Excel 应用程序实例
    print(i.pid)
    # 打印当前 Excel 应用程序的 PID

    for j in i.books:
        # 遍历当前 Excel 应用程序打开的所有工作簿
        print()
        print(j.name)
        # 打印当前工作簿的名称

# 打印当前活动单元格所在的 Excel 应用程序的 PID
rng.sheet.book.app.pid

# 导入 Python 的 time 模块，用于时间延迟
import time

# 设置延迟 4 秒，确保 Excel 实例稳定
time.sleep(4)

# 关闭由 excelpid 函数返回的 Excel 应用程序实例
xlwings.apps[excelpid()].quit()


aaa = '工资单_workbook_xls__20250522140431350343'
aaa

bbb = r'https://d.docs.live.net/9122e41a29eea899/sb_yufengguang/xls/t6/%E5%B7%A5%E8%B5%84%E5%8D%95_workbook_xls__20250522140431350343.xlsx'
bbb

#import f_excel.over.open_and_process_excel as ov


#ov.open_and_process_excel(bbb)


import frankyu.xlsx.url_3 as ur
ur.url(bbb,aaa)


import os
import subprocess
import locale
from typing import Tuple

def remove_files_safely(path: str = "123") -> Tuple[int, str, str]:
    """
    安全地删除指定路径下的文件或目录。
    
    参数:
        path (str): 要删除的文件或目录的路径，默认为"123"。
        
    返回值:
        Tuple[int, str, str]: (退出代码, 标准输出, 标准错误)。
    """
    # 检查提供的路径是否存在
    if not os.path.exists(path):
        return -1, "", f"指定的路径 {path} 不存在。"

    # 构建删除命令
    if os.path.isfile(path):
        command = ["del", '/F', '/Q', path]
    elif os.path.isdir(path):
        command = ["rmdir", '/S', '/Q', path]
    else:
        return -1, "", f"无法识别的路径类型: {path}"

    def execute_command(command) -> Tuple[int, str, str]:
        """
        在 shell 中执行命令，并返回退出代码、标准输出和标准错误。
        """
        print(f"运行的命令: {' '.join(command)}")
        
        try:
            result = subprocess.run(
                command,
                capture_output=True,
                text=False,
                check=False,
                shell=True,
            )
            
            def decode_with_fallback(byte_data):
                """尝试 UTF-8 解码，如果失败则使用系统默认编码解码"""
                if not byte_data:
                    return ""
                try:
                    return byte_data.decode('utf-8').strip()
                except UnicodeDecodeError:
                    sys_encoding = locale.getpreferredencoding()
                    return byte_data.decode(sys_encoding, errors='replace').strip()

            stdout = decode_with_fallback(result.stdout)
            stderr = decode_with_fallback(result.stderr)

            print(f"命令输出 (stdout):\n{stdout}")
            print(f"命令错误 (stderr):\n{stderr}" if stderr else "命令错误 (stderr): 无")

            return result.returncode, stdout, stderr

        except Exception as e:
            print(f"执行命令时发生异常: {str(e)}")
            return -1, "", str(e)

    # 尝试删除文件或目录
    ret_code, stdout, stderr = execute_command(command)
    
    # 如果删除失败，尝试终止占用进程
    if ret_code != 0 and "cannot access the file because it is being used by another process" in stderr.lower():
        # 获取占用文件的进程ID
        tasklist_command = ['tasklist', '/FI', f'PATH eq {path}', '/FO', 'CSV']
        ret_code_tasklist, stdout_tasklist, stderr_tasklist = execute_command(tasklist_command)
        
        if ret_code_tasklist == 0:
            lines = stdout_tasklist.split('\n')
            if len(lines) > 1:
                for line in lines[1:]:
                    parts = line.split(',')
                    if len(parts) >= 2:
                        pid = parts[1].strip('"')
                        # 终止进程
                        kill_command = ['taskkill', '/PID', pid, '/F']
                        ret_code_kill, stdout_kill, stderr_kill = execute_command(kill_command)
                        
                        if ret_code_kill == 0:
                            print(f"成功终止进程 ID: {pid}")
                            # 再次尝试删除文件或目录
                            ret_code, stdout, stderr = execute_command(command)
                            break
    
    return ret_code, stdout, stderr

# 示例调用
if __name__ == "__main__":
    ret_code, stdout, stderr = remove_files_safely("12345")
    print(f"操作结果: 退出代码={ret_code}, 标准输出={stdout}, 标准错误={stderr}")






aaa = '世界简史 workbook_xls__20250623094705329139'
aaa

bbb = r'https://d.docs.live.net/9122e41a29eea899/sb_yufengguang/xls/%E4%B8%96%E7%95%8C%E7%AE%80%E5%8F%B2%20workbook_xls__20250623094705329139.xlsx'
bbb

#import f_excel.over.open_and_process_excel as ov


#ov.open_and_process_excel(bbb)


import frankyu.xlsx.url_3 as ur
ur.url(bbb,aaa)


# -*- coding: utf-8 -*-
"""
Excel自動化處理腳本

功能架構：
1. 核心功能模組
    - 工作表複製
    - 結構化表格創建 (將數據範圍轉換為Excel Table)
    - 公式列添加 (在表格中新增列並填充公式)
    - 數據透視表生成 (基於表格數據創建數據透視表)
    - 數據後處理 (配置透視表欄位、格式化)

設計原則：
1. 模組化設計 - 每個功能獨立封裝成函數，提高可讀性和重用性。
2. 防禦式程式設計 - 包含檔案是否存在檢查、錯誤捕獲等，增強腳本的穩定性。
3. 資源管理 - 使用try...finally塊確保Excel應用程式和工作簿在執行完畢或出錯後能被正確關閉和釋放，避免資源洩漏。
4. 類型提示 (Type Hinting) - 使用Python的類型提示功能（如`: str`, `-> win32.CDispatch`），幫助理解函數預期接收和返回的數據類型，提高程式碼可讀性和可維護性。
5. 常量集中管理 - 將檔案路徑、工作表名稱、表格名稱等常用設定放在腳本開頭的常量區，方便修改和管理。

技術要點：
- 使用win32com庫實現Excel COM自動化，直接調用Excel應用程式的API來控制其行為。
- 完全模擬GUI操作流程 - win32com的操作邏輯很大程度上反映了在Excel使用者介面中的步驟（例如，創建PivotCache，然後基於Cache創建PivotTable）。
- 支援大範圍數據處理(13,900行) - COM自動化通常比直接讀寫檔案（如openpyxl, pandas）在處理特定複雜格式和操作（如透視表、結構化表格）時更具優勢，尤其適用於已經存在大量公式或特定格式的檔案。
- 自動化公式填充 - 利用Excel結構化表格的特性，在表格列中添加公式，該公式會自動應用於表格的所有行。
- 動態數據透視表配置 - 通過程式碼設定數據透視表的數據源、行列欄位和值欄位。

前置條件：
- 系統上安裝了Microsoft Excel。
- 安裝了Python環境。
- 安裝了win32com庫 (`pip install pywin32`)。

注意事項：
- COM自動化依賴於Excel的安裝，且可能會受到Excel版本差異的影響。
- 在腳本執行期間，Excel視窗可能會彈出（如果`visible=True`），請勿手動干擾。
- 處理大型檔案時，COM操作可能會比較耗時。
- 錯誤處理機制會嘗試捕獲COM錯誤，但某些底層COM異常可能需要更複雜的處理。
"""

import win32com.client as win32 # 導入win32com.client模組，用於與COM對象互動，通常用於Windows應用程式自動化，這裡用於控制Excel。
import os # 導入os模組，用於作業系統相關操作，這裡用於檢查檔案是否存在。
import traceback # 導入traceback模組，用於獲取詳細的錯誤堆疊資訊，方便除錯。
from typing import Optional, Tuple # 從typing模組導入Optional和Tuple，用於提供類型提示，增強程式碼可讀性。

# === 常量定義區 ===
# 集中管理腳本中使用的各種設定值。
DEFAULT_WORKBOOK_PATH = r"D:\3.xlsx" # 預設的工作簿檔案路徑。使用原始字串(r"...")避免反斜線的轉義問題。
DEFAULT_SHEET_NAME = "B234-581" # 預設要複製的工作表名稱。
DEFAULT_TABLE_NAME = "t1" # 預設創建的結構化表格(ListObject)的名稱。
DEFAULT_PIVOT_NAME = "t3" # 預設創建的數據透視表的名稱。
DEFAULT_DATE_STR = "2025.04.27" # 預設日期字串，程式碼中未使用，可能保留用於其他目的。
XL_COUNT = -4112 # Excel內建常量，表示COUNT函數，用於數據透視表的值欄位匯總方式。這是通過COM介面訪問Excel功能時需要使用的特定值。
XL_DATABASE = 1 # Excel內建常量，表示數據透視表數據源類型為數據庫或清單。
XL_ROW_FIELD = 1 # Excel內建常量，表示將字段添加到數據透視表的行區域。

# === 核心功能函數 ===


def add_formula_columns(worksheet: win32.CDispatch, len_col: str = "K", count_col: str = "L", ref_col: str = "SN_NO",aaa = -7) -> None:
    """
    在工作表中指定的列添加標題和公式。

    此函數通常用於在一個已經創建了結構化表格的工作表上添加計算列。
    由於是在結構化表格所在的列設置公式，Excel會自動將公式應用於表格的所有數據行。

    Args:
        worksheet (win32.CDispatch): 要添加列和公式的工作表的COM對象。
        len_col (str, optional): 用於存放長度計算公式的新列的列字母標識 (例如 "K")。
                                 預設為 "K"。
        count_col (str, optional): 用於存放計數計算公式的新列的列字母標識 (例如 "L")。
                                   預設為 "L"。
        ref_col (str, optional): 用於公式計算的參考列的標題名稱 (例如 "SN_NO")。
                                 預設為 "SN_NO"。

    Returns:
        None: 函數執行操作，沒有明確返回值。
    """
    # 在K1單元格設置列標題為"len"。
    worksheet.Range(f"{len_col}1").Value = "len"
    # 在L1單元格設置列標題為"cou"。
    worksheet.Range(f"{count_col}1").Value = "cou"
    # 在K2單元格設置公式。FormulaR1C1使用R1C1引用樣式。
    # f"=LEN([@[{ref_col}]])" 是結構化表格中引用同一行 ref_col 列的值的公式。
    # Excel會自動將此公式應用於 len_col 列的每一行。
    worksheet.Range(f"{len_col}2").FormulaR1C1 = f"=LEN([@[{ref_col}]])"
    # 在L2單元格設置公式。
    # "=COUNTIFS(C[-7],[@[{ref_col}]])" 是使用COUNTIFS函數進行條件計數的公式。
    # C[-7] 表示相對於當前單元格 (count_col 列) 左邊第七列的整列。
    # [@[{ref_col}]] 引用同一行 ref_col 列的值作為計數條件。
    # 這個公式的目的是計算 ref_col 列的值在 C[-7] 列中出現的次數。
    # Excel會自動將此公式應用於 count_col 列的每一行。
    worksheet.Range(f"{count_col}2").FormulaR1C1 = f"=COUNTIFS(C[{aaa}],[@[{ref_col}]])"
    # 沒有明確的返回值，函數完成其操作即可。


import streamlit as st

st.title("文本展示示例")
st.header("这是一个二级标题")
st.subheader("这是一个三级标题")

st.write("---") # 分割线

st.write("`st.write()` 是一个非常灵活的函数，可以显示多种内容。")
st.markdown("你可以在 Markdown 中使用 **粗体**、*斜体* 和 [链接](https://streamlit.io)。")
st.caption("这是一个小标题，常用于备注。")

st.code("""
import pandas as pd
df = pd.DataFrame({'col1': [1, 2], 'col2': [3, 4]})
st.write(df)
""", language="python")

st.latex(r"""
E=mc^2
""")


import win32com.client  # 导入
import pythoncom  # 导入
import sys  # 导入
import traceback  # 追溯

# Excel 常數 (constant) (對應 (correspond)
# 到 VBA 常數)
xlContinuous = 1  # xlContinuous (連續的)
xlThin = 2  # xlThin (細的)
xlExpression = 1  # xlExpression (表達式)
xlLeft = 1  # xlLeft (左)
xlRight = 2  # xlRight (右)
xlTop = 3  # xlTop (頂部/上)
xlBottom = 4  # xlBottom (底部/下)
xlThemeColorDark1 = 1  # xlThemeColorDark1 (主題顏色深 1)


def initialize_excel(  # 定義函數 (function) 初始化 (initialize)
    # Excel
    app_name="Excel.Application",  # app_name: 指定要連接或
    # 啟動的 Excel 應用程式的名稱，預設為 "Excel.Application"。
    visible=True  # visible: 一個布林值，指示 Excel
    # 應用程式是否應該在啟動時可見，預設為 True。
):
    """
    初始化 (initialize) Excel COM 物件 (object)，並返回 (return)
    Excel 實例 (instance)、工作簿 (workbook) 和工作表 (worksheet)。
    """
    # 思路:
    # 1. 嘗試初始化 COM 元件 (pythoncom.CoInitialize)。
    # 2. 嘗試獲取一個正在運行的 Excel 應用程式實例
    #    (win32com.client.GetActiveObject)。
    # 3. 如果沒有正在運行的實例，則啟動一個新的 Excel
    #    應用程式實例 (win32com.client.Dispatch)。
    # 4. 檢查 Excel 實例是否成功獲取或啟動。
    # 5. 設置 Excel 應用程式的可見性並禁用警告提示。
    # 6. 嘗試獲取活動的工作簿 (excel.ActiveWorkbook)。
    #    如果沒有活動的工作簿，則創建一個新的工作簿。
    # 7. 嘗試獲取活動的工作表 (workbook.ActiveSheet)。
    # 8. 如果所有步驟都成功，則返回 Excel 應用程式實例、
    #    工作簿和工作表。
    # 9. 在任何步驟中如果發生錯誤，則打印錯誤信息並返回 None。
    try:
        pythoncom.CoInitialize()  # 初始化 (initialize) COM
        print("COM 已初始化 (initialized)。")  # 打印

        try:
            # 嘗試 (try) 獲取 (get) 活動 (active) 的 Excel
            # 實例 (instance)
            excel = win32com.client.GetActiveObject(app_name)
            # 獲取活動對象 (get active object)
        except pythoncom.com_error:  # 捕捉 (catch) COM 錯誤 (error)
            # 如果 (if) 沒有 (no) 活動 (active) 的實例
            # (instance)，啟動 (launch) 一個新 (new) 實例 (instance)
            print(
                f"無法 (cannot) 找到 (find) 活動 (active) 的 "
                f"{app_name} 實例 (instance)，啟動 (launch) 新 (new) "
                f"實例 (instance)。"
            )  # 打印
            excel = win32com.client.Dispatch(app_name)
            # 創建對象 (create object)

        # 確保 (ensure) Excel 被 (be) 正確地 (correctly)
        # 啟動 (launched)
        if excel is None:  # 如果 (if) excel 是 None
            print(
                f"無法 (cannot) 啟動 (launch) 或 (or) 訪問 (access) "
                f"{app_name} 應用程序 (application)。"
            )  # 打印
            return None  # 返回 (return) None

        # 設置 (set) Excel 為 (to) 前台 (foreground) 顯示
        # (display) 並 (and) 取消 (cancel) 提醒 (alerts)
        excel.Visible = visible  # 可見性 (visibility)
        excel.DisplayAlerts = False  # 顯示警告 (display alerts)
        print("Excel 已設置 (set) 為 (to) 前台 (foreground) 顯示 "
              "(display)，並 (and) 取消 (canceled) 提醒 (alerts)。")  # 打印

        try:
            # 嘗試 (try) 獲取 (get) 活動 (active) 工作簿 (workbook)
            workbook = excel.ActiveWorkbook  # 活動工作簿 (active workbook)
            if workbook is None:  # 如果 (if) workbook 是 None
                print("沒有 (no) 活動 (active) 的工作簿 (workbook)，"
                      "創建 (create) 一個新 (new) 工作簿 (workbook)。")  # 打印
                workbook = excel.Workbooks.Add()  # 添加工作簿 (add workbook)
        except Exception as e:  # 捕捉 (catch) 異常 (exception)
            print(
                f"取得 (get) 活動 (active) 工作簿 (workbook) 時 "
                f"(when) 發生 (occurred) 錯誤 (error): {e}"
            )  # 打印
            traceback.print_exc()  # 打印異常信息 (print exception information)
            return None  # 返回 (return) None

        try:
            # 嘗試 (try) 獲取 (get) 活動 (active) 工作表
            # (worksheet)
            sheet = workbook.ActiveSheet  # 活動工作表 (active worksheet)
            if sheet is None:  # 如果 (if) sheet 是 None
                print(
                    "沒有 (no) 活動 (active) 的工作表 (worksheet)，請 (please) "
                    "確認 (confirm) 工作簿 (workbook) 包含 (contains) "
                    "有效 (valid) 的工作表 (worksheet)。"
                )  # 打印
                return None  # 返回 (return) None
        except Exception as e:  # 捕捉 (catch) 異常 (exception)
            print(
                f"取得 (get) 活動 (active) 工作表 (worksheet) 時 "
                f"(when) 發生 (occurred) 錯誤 (error): {e}"
            )  # 打印
            traceback.print_exc()  # 打印異常信息 (print exception information)
            return None  # 返回 (return) None

        print("Excel 初始化 (initialized) 成功 (successfully)。")  # 打印
        return excel, workbook, sheet  # 返回 (return)

    except Exception as e:  # 捕捉 (catch) 異常 (exception)
        print(
            f"初始化 (initialization) 過程中 (process) 發生 (occurred) "
            f"未知 (unknown) 錯誤 (error): {e}"
        )  # 打印
        traceback.print_exc()  # 打印異常信息 (print exception information)
        return None  # 返回 (return) None


def clear_conditional_formatting(  # 定義函數 (function) 清除 (clear)
    # 條件式格式設定 (conditional formatting)
    sheet  # sheet: 要清除條件式格式設定的工作表物件。
):
    """
    清除 (clear) 指定 (specified) 工作表 (worksheet) 中的 (in) 所有
    (all) 條件式格式設定 (conditional formatting)。
    """
    # 思路:
    # 1. 使用傳入的工作表物件 (sheet)。
    # 2. 訪問工作表中的所有單元格 (sheet.Cells)。
    # 3. 訪問單元格的格式條件集合 (.FormatConditions)。
    # 4. 調用集合的 Delete 方法來移除所有條件式格式設定。
    # 5. 如果操作成功，則打印成功的消息。
    # 6. 如果發生任何異常，則捕獲異常，打印錯誤消息和異常追溯信息。
    try:
        sheet.Cells.FormatConditions.Delete()  # 刪除 (delete) 格式條件
        # (format conditions)
        print("已清除 (cleared) 所有 (all) 條件式格式設定 "
              "(conditional formatting)。")  # 打印
    except Exception as e:  # 捕捉 (catch) 異常 (exception)
        print(
            f"清除 (clearing) 條件式格式設定 (conditional "
            f"formatting) 時 (when) 發生 (occurred) 錯誤 (error): {e}"
        )  # 打印
        traceback.print_exc()  # 打印異常信息 (print exception information)


def apply_border_conditional_formatting(  # 定義函數 (function) 應用 (apply)
    # 邊框 (border) 條件式格式設定 (conditional formatting)
    target_range,  # target_range: 要應用條件式格式設定的 Excel
    # 範圍物件。
    formula="=NOT(OR(A1=\"\",A1=0))"  # formula: 用於判斷是否應用邊框的
    # Excel 公式，預設為 "=NOT(OR(A1=\"\",A1=0))"，表示當 A1 不為空且不為 0 時應用邊框。
):
    """
    為 (for) 指定 (specified) 的範圍 (range) 加入 (add) 條件式格式設定
    (conditional formatting)，並 (and) 應用 (apply) 邊框樣式 (border style)。
    """
    # 思路:
    # 1. 從 win32com.client 導入 constants 以使用 Excel 常量。
    # 2. 檢查目標範圍是否包含合併的單元格，如果是則打印消息並返回，
    #    因為在合併的單元格上應用條件式格式設定可能會導致問題。
    # 3. 使用目標範圍的 FormatConditions.Add 方法添加一個新的條件式
    #    格式設定規則，類型設置為基於公式 (xlExpression)，公式使用傳入的 formula。
    # 4. 將新創建的格式條件設置為第一個優先級。
    # 5. 遍歷邊框的四個邊 (左、右、上、下)。
    # 6. 對於每個邊，獲取其邊框物件，並設置線條樣式為連續 (xlContinuous)
    #    和粗細為細 (xlThin)。
    # 7. 如果操作成功，則打印成功的消息。
    # 8. 如果發生任何異常，則捕獲異常，打印錯誤消息和異常追溯信息。
    try:
        from win32com.client import constants  # 從 (from) win32com.client
        # 導入 (import) 常數 (constants)
        if target_range.MergeCells:  # 如果 (if) 目標範圍 (target range)
            # 包含合併儲存格 (merge cells)
            print(
                "目標範圍 (target range) 包含 (contains) 合併儲存格 "
                "(merged cells)，請 (please) 確認 (confirm) 並 (and) "
                "重試 (retry)。"
            )  # 打印
            return  # 返回 (return)

        format_condition = target_range.FormatConditions.Add(  # 添加 (add)
            # 格式條件 (format condition)
            Type=constants.xlExpression,  # Type: 條件式格式設定的類型，
            # 此處使用 xlExpression 表示基於公式。
            Formula1=formula  # Formula1: 條件式格式設定的公式。
        )
        format_condition.SetFirstPriority()  # 設置 (set) 首要 (first)
        # 優先級 (priority)
        print(
            f"已加入 (added) 條件式格式設定 (conditional "
            f"formatting) 規則 (rule) (邊框 (border))，公式 (formula): "
            f"{formula}。"
        )  # 打印

        for border_type in [xlLeft, xlRight, xlTop, xlBottom]:  # 遍歷
            # (iterate) 邊框類型 (border type)
            border = format_condition.Borders(border_type)
            # border_type: 指定要設定邊框的邊，可以是 xlLeft、xlRight、
            # xlTop 或 xlBottom。
            border.LineStyle = xlContinuous  # LineStyle: 邊框的線條樣式，
            # 此處使用 xlContinuous 表示實線。
            border.Weight = xlThin  # Weight: 邊框的粗細，此處使用 xlThin
            # 表示細線。
        print("已為 (for) 條件式格式設定 (conditional formatting) 規則 "
              "(rule) 應用 (applied) 邊框 (border)。")  # 打印
    except Exception as e:  # 捕捉 (catch) 異常 (exception)
        print(
            f"應用 (applying) 邊框 (border) 條件式格式設定 "
            f"(conditional formatting) 時 (when) 發生 (occurred) 錯誤 "
            f"(error): {e}"
        )  # 打印
        traceback.print_exc()  # 打印異常信息 (print exception information)


def apply_font_conditional_formatting(  # 定義函數 (function) 應用 (apply)
    # 字體 (font) 條件式格式設定 (conditional formatting)
    target_range,  # target_range: 要應用條件式格式設定的 Excel
    # 範圍物件。
    formula="=OR(A1=\"\",A1=0)"  # formula: 用於判斷是否應用字體格式的
    # Excel 公式，預設為 "=OR(A1=\"\",A1=0)"，表示當 A1 為空或為 0 時應用字體格式。
):
    """
    為 (for) 指定 (specified) 的範圍 (range) 加入 (add) 條件式格式設定
    (conditional formatting)，並 (and) 應用 (apply) 字體顏色 (font color)。
    """
    # 思路:
    # 1. 從 win32com.client 導入 constants 以使用 Excel 常量。
    # 2. 檢查目標範圍是否包含合併的單元格，如果是則打印消息並返回，
    #    因為在合併的單元格上應用條件式格式設定可能會導致問題。
    # 3. 使用目標範圍的 FormatConditions.Add 方法添加一個新的條件式
    #    格式設定規則，類型設置為基於公式 (xlExpression)，公式使用傳入的 formula。
    # 4. 將新創建的格式條件設置為第一個優先級。
    # 5. 獲取格式條件的字體物件。
    # 6. 設置字體的顏色為主題顏色 Dark1 (xlThemeColorDark1)。
    # 7. 如果操作成功，則打印成功的消息。
    # 8. 如果發生任何異常，則捕獲異常，打印錯誤消息和異常追溯信息。
    try:
        from win32com.client import constants  # 從 (from) win32com.client
        # 導入 (import) 常數 (constants)
        if target_range.MergeCells:  # 如果 (if) 目標範圍 (target range)
            # 包含合併儲存格 (merge cells)
            print(
                "目標範圍 (target range) 包含 (contains) 合併儲存格 "
                "(merged cells)，請 (please) 確認 (confirm) 並 (and) "
                "重試 (retry)。"
            )  # 打印
            return  # 返回 (return)

        format_condition = target_range.FormatConditions.Add(  # 添加 (add)
            # 格式條件 (format condition)
            Type=constants.xlExpression,  # Type: 條件式格式設定的類型，
            # 此處使用 xlExpression 表示基於公式。
            Formula1=formula  # Formula1: 條件式格式設定的公式。
        )
        format_condition.SetFirstPriority()  # 設置 (set) 首要 (first)
        # 優先級 (priority)
        print(
            f"已加入 (added) 條件式格式設定 (conditional "
            f"formatting) 規則 (rule) (字體顏色 (font color))，公式 "
            f"(formula): {formula}。"
        )  # 打印

        font = format_condition.Font  # 字體 (font)
        font.ThemeColor = xlThemeColorDark1  # ThemeColor: 字體的顏色，
        # 此處使用主題顏色 Dark1。
        print("已為 (for) 條件式格式設定 (conditional formatting) 規則 "
              "(rule) 應用 (applied) 字體顏色 (font color)。")  # 打印
    except Exception as e:  # 捕捉 (catch) 異常 (exception)
        print(
            f"應用 (applying) 字體顏色 (font color) 條件式格式設定 "
            f"(conditional formatting) 時 (when) 發生 (occurred) 錯誤 "
            f"(error): {e}"
        )  # 打印
        traceback.print_exc()  # 打印異常信息 (print exception information)


def set_cell_value_and_select(  # 定義函數 (function) 設置 (set) 儲存格值
    # (cell value) 並 (and) 選取 (select)
    sheet,  # sheet: 要操作的工作表物件。
    value_cell="N12",  # value_cell: 要設定值的目標儲存格地址，
    # 預設為 "N12"。
    value_to_set=0,  # value_to_set: 要設定到目標儲存格的值，
    # 預設為 0。
    select_cell="K7"  # select_cell: 設定值後要選取的儲存格地址，
    # 預設為 "K7"。
):
    """
    設定 (set) 指定 (specified) 儲存格 (cell) 的值 (value)，並 (and)
    選取 (select) 另一個 (another) 儲存格 (cell)。
    """
    # 思路:
    # 1. 使用傳入的工作表物件 (sheet)。
    # 2. 使用 Range 方法指定要設定值的儲存格 (value_cell)，
    #    並將其 Value 屬性設置為傳入的 value_to_set。
    # 3. 使用 Range 方法指定要選取的儲存格 (select_cell)，
    #    並調用其 Select 方法來選取該儲存格。
    # 4. 如果操作成功，則打印成功的消息。
    # 5. 如果發生任何異常，則捕獲異常，打印錯誤消息和異常追溯信息。
    try:
        sheet.Range(value_cell).Value = value_to_set  # 設定值 (set value)
        print(f"已將 (has set) 儲存格 (cell) {value_cell} 的值 (value) "
              f"設定 (set) 為 (to) {value_to_set}。")  # 打印
        sheet.Range(select_cell).Select()  # 選取 (select)
        print(f"已選取 (selected) 儲存格 (cell) {select_cell}。")  # 打印
    except Exception as e:  # 捕捉 (catch) 異常 (exception)
        print(
            f"設定 (setting) 儲存格值 (cell value) 或 (or) 選取 "
            f"(selecting) 儲存格 (cell) 時 (when) 發生 (occurred) 錯誤 "
            f"(error): {e}"
        )  # 打印
        traceback.print_exc()  # 打印異常信息 (print exception information)


def run_excel_conditional_formatting(  # 定義函數 (function) 運行 (run) Excel
    # 條件式格式設定 (conditional formatting)
    target_range_address="A1:AA219"  # target_range_address: 要應用條件式
    # 格式設定的目標範圍地址，預設為 "A1:AA219"。
):
    """
    主函數 (main function)，協調 (coordinate) 執行 (execute) 所有 (all)
    條件式格式設定 (conditional formatting) 操作 (operation)。
    """
    # 思路:
    # 1. 調用 initialize_excel 函數來獲取 Excel 應用程式、工作簿和工作表的物件。
    # 2. 如果初始化失敗 (返回 None)，則打印錯誤消息並終止程序。
    # 3. 解包 initialize_excel 返回的 Excel 物件。
    # 4. 調用 clear_conditional_formatting 函數清除工作表中的所有現有條件式格式設定。
    # 5. 使用工作表的 Range 方法根據傳入的 target_range_address 創建目標範圍物件。
    # 6. 調用 apply_border_conditional_formatting 函數將邊框條件式格式設定應用於目標範圍。
    # 7. 調用 apply_font_conditional_formatting 函數將字體顏色條件式格式設定應用於目標範圍。
    # 8. 調用 set_cell_value_and_select 函數設置特定的儲存格值並選取另一個儲存格。
    # 9. 如果所有步驟都成功完成，則打印成功的消息。
    # 10. 如果在執行過程中發生任何異常，則捕獲異常，打印錯誤消息和異常追溯信息。
    # 11. 在 finally 塊中，嘗試解除初始化 COM 組件 (pythoncom.CoUninitialize)。
    excel_objects = initialize_excel()  # 初始化 (initialize) Excel
    if not excel_objects:  # 如果 (if) excel_objects 不是 (not) 真 (true)
        print("初始化 (initialization) 失敗 (failed)，程序 (program) 終止 "
              "(terminated)。")  # 打印
        sys.exit(1)  # 退出 (exit) 程序 (program)

    excel, workbook, sheet = excel_objects  # 解包 (unpack) Excel 對象
    # (objects)
    try:
        clear_conditional_formatting(sheet)  # 清除 (clear) 條件式格式設定
        # (conditional formatting)

        target_range = sheet.Range(target_range_address)  # 目標範圍
        # (target range)
        print(f"目標範圍 (target range) 設定 (set) 為 (to): "
              f"{target_range.Address}")  # 打印

        apply_border_conditional_formatting(target_range)
        # 應用 (apply) 邊框 (border) 條件式格式設定 (conditional formatting)
        apply_font_conditional_formatting(target_range)
        # 應用 (apply) 字體 (font) 條件式格式設定 (conditional formatting)

        set_cell_value_and_select(sheet)  # 設置 (set) 儲存格值 (cell value)
        # 並 (and) 選取 (select)

        print("腳本 (script) 執行 (executed) 成功 (successfully)。")  # 打印
    except Exception as e:  # 捕捉 (catch) 異常 (exception)
        print(
            f"執行 (executing) 主 (main) 操作 (operation) 時 (when) "
            f"發生 (occurred) 錯誤 (error): {e}"
        )  # 打印
        traceback.print_exc()  # 打印異常信息 (print exception information)
    finally:
        try:
            pass
            # excel.Quit()  # 退出 (quit) Excel
            pythoncom.CoUninitialize()  # 解除 (uninitialize) COM
            print("Excel 已關閉 (closed)，COM 已解除 (uninitialized)。")  # 打印
        except Exception as e:  # 捕捉 (catch) 異常 (exception)
            print(
                f"關閉 (closing) Excel 或 (or) 解除 (uninitializing) COM "
                f"時 (when) 發生 (occurred) 錯誤 (error): {e}"
            )  # 打印
            traceback.print_exc()  # 打印異常信息 (print exception information)


if __name__ == "__main__":  # 如果 (if) 是主模塊 (main module)
    run_excel_conditional_formatting()  # 運行 (run) Excel 條件式格式設定
    # (conditional formatting)


import os
import tempfile
import streamlit as st
from docx import Document
from pptx import Presentation
import pdfplumber

def save_uploaded_file(uploaded_file):
    """保存上传的文件到临时文件"""
    try:
        with tempfile.NamedTemporaryFile(delete=False, suffix=os.path.splitext(uploaded_file.name)[1]) as tmp_file:
            tmp_file.write(uploaded_file.getvalue())
            return tmp_file.name
    except Exception as e:
        st.error(f"文件保存失败: {str(e)}")
        return None

def extract_text(file_path):
    """通用文本提取函数"""
    try:
        file_path_lower = file_path.lower()
        if file_path_lower.endswith('.pdf'):
            with pdfplumber.open(file_path) as pdf:
                return "\n".join([p.extract_text() for p in pdf.pages if p.extract_text()])
        elif file_path_lower.endswith('.docx'):
            doc = Document(file_path)
            return "\n".join([p.text for p in doc.paragraphs if p.text])
        elif file_path_lower.endswith('.pptx'):
            prs = Presentation(file_path)
            text = []
            for slide in prs.slides:
                for shape in slide.shapes:
                    if hasattr(shape, "text"):
                        text.append(shape.text)
            return "\n".join(text)
        elif file_path_lower.endswith('.txt'):
            with open(file_path, 'r', encoding='utf-8', errors='ignore') as f:
                return f.read()
        else:
            st.warning(f"不支持的文件类型: {os.path.splitext(file_path)[1]}")
            return ""
    except Exception as e:
        st.error(f"提取失败: {str(e)}")
        return ""

def get_file_list(directory, extensions=['.pdf', '.docx', '.pptx', '.txt']):
    """获取目录中指定扩展名的文件列表"""
    file_list = []
    if os.path.exists(directory):
        for root, _, files in os.walk(directory):
            for file in files:
                if any(file.lower().endswith(ext) for ext in extensions):
                    file_list.append(os.path.join(root, file))
    return sorted(file_list)

def main():
    st.title("📄 文件文字提取工具")
    
    # 创建选项卡
    tab1, tab2 = st.tabs(["上传文件", "指定路径"])
    
    with tab1:
        uploaded_file = st.file_uploader("上传文件", type=['pdf', 'docx', 'pptx', 'txt'], key="uploader")
        
        if uploaded_file:
            # 保存上传的文件到临时位置
            temp_file_path = save_uploaded_file(uploaded_file)
            
            if temp_file_path:
                with st.spinner("正在提取文字..."):
                    text = extract_text(temp_file_path)
                
                # 清理临时文件
                try:
                    os.unlink(temp_file_path)
                except:
                    pass
                
                if text:
                    st.success("提取完成！")
                    file_name = uploaded_file.name.split('.')[0] + ".txt"
                    st.download_button(
                        "下载文本",
                        text,
                        file_name,
                        key="upload_download"
                    )
                    with st.expander("预览内容", expanded=True):
                        st.text(text[:2000] + ("..." if len(text)>2000 else ""))
    
    with tab2:
        default_path = os.path.expanduser("~/360a")
        input_path = st.text_input("输入文件路径或目录", value=default_path)
        
        if st.button("扫描文件", key="scan_files"):
            if os.path.isfile(input_path):
                st.session_state.file_list = [input_path]
                st.session_state.selected_file = input_path
            elif os.path.isdir(input_path):
                st.session_state.file_list = get_file_list(input_path)
                st.session_state.selected_file = None
            else:
                st.error("路径不存在，请输入有效的文件或目录路径")
        
        if 'file_list' in st.session_state and st.session_state.file_list:
            selected_file = st.selectbox(
                "选择要提取的文件",
                options=st.session_state.file_list,
                index=0,
                key="file_selector"
            )
            
            if st.button("提取文本", key="extract_path"):
                with st.spinner("正在提取文字..."):
                    text = extract_text(selected_file)
                
                if text:
                    st.success("提取完成！")
                    file_name = os.path.basename(selected_file).split('.')[0] + ".txt"
                    st.download_button(
                        "下载文本",
                        text,
                        file_name,
                        key="path_download"
                    )
                    with st.expander("预览内容", expanded=True):
                        st.text(text[:2000] + ("..." if len(text)>2000 else ""))
        elif 'file_list' in st.session_state:
            st.warning("未找到符合条件的文件")

if __name__ == "__main__":
    main()


import streamlit as st

st.title("表单示例")

with st.form("my_form"):
    st.write("在表单中输入你的信息")
    name = st.text_input("姓名")
    email = st.text_input("邮箱")
    message = st.text_area("留言")

    # Every form must have a submit button.
    submitted = st.form_submit_button("提交")

    if submitted:
        st.write("表单已提交！")
        st.write(f"姓名: {name}")
        st.write(f"邮箱: {email}")
        st.write(f"留言: {message}")


"""
跨平台系统音量控制工具(交互式环境友好版)

功能：
- 自动检测是否在交互式环境中运行
- Windows: 使用pycaw库控制音量
- macOS: 使用osascript调用AppleScript控制音量
- Linux: 使用alsamixer/amixer控制音量(PulseAudio/ALSA)
- 提供统一的错误处理和安装指导
"""

import sys
import platform
import subprocess
from typing import Optional, Tuple

# 检测是否在交互式环境中运行(如Jupyter/IPython)
IS_INTERACTIVE = hasattr(sys, 'ps1') or bool(getattr(sys, 'base_prefix', None))

# 全局配置
SUPPORTED_PLATFORMS = ['Windows', 'Darwin', 'Linux']
CURRENT_PLATFORM = platform.system()

class VolumeControlError(Exception):
    """自定义音量控制异常基类"""
    pass

class UnsupportedPlatformError(VolumeControlError):
    """不支持的平台异常"""
    pass

class InvalidVolumeLevelError(VolumeControlError):
    """无效的音量级别异常"""
    pass

class VolumeControl:
    def __init__(self, default_volume: float = 0.5):
        """
        初始化音量控制器
        
        Args:
            default_volume: 默认音量级别(0.0-1.0), 在未设置音量时作为默认值
        """
        self.default_volume = default_volume
        self.initialized = False
        self.platform_specific_init()
    
    def platform_specific_init(self):
        """平台特定的初始化"""
        try:
            if CURRENT_PLATFORM == 'Windows':
                self.init_windows()
            elif CURRENT_PLATFORM == 'Darwin':
                self.init_macos()
            elif CURRENT_PLATFORM == 'Linux':
                self.init_linux()
            else:
                raise UnsupportedPlatformError(f"不支持的平台: {CURRENT_PLATFORM}")
            
            self.initialized = True
        except Exception as e:
            print(f"初始化失败: {str(e)}", file=sys.stderr)
            self.initialized = False
    
    def init_windows(self):
        """Windows平台初始化"""
        try:
            from pycaw.pycaw import AudioUtilities, IAudioEndpointVolume
            from ctypes import cast, POINTER
            from comtypes import CLSCTX_ALL
            
            devices = AudioUtilities.GetSpeakers()
            interface = devices.Activate(
                IAudioEndpointVolume._iid_, CLSCTX_ALL, None)
            self.volume_interface = cast(interface, POINTER(IAudioEndpointVolume))
        except ImportError:
            raise ImportError(
                "缺少pycaw库。请使用以下命令安装:\n"
                "pip install pycaw\n"
                "或访问 https://github.com/AndreMiras/pycaw")
        except Exception as e:
            raise VolumeControlError(f"Windows音频接口初始化失败: {str(e)}")
    
    def init_macos(self):
        """macOS平台初始化"""
        try:
            subprocess.run(['which', 'osascript'], check=True, 
                          stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        except subprocess.CalledProcessError:
            raise VolumeControlError("macOS: 未找到osascript命令")
    
    def init_linux(self):
        """Linux平台初始化"""
        try:
            subprocess.run(['which', 'amixer'], check=True,
                         stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        except subprocess.CalledProcessError:
            raise VolumeControlError(
                "Linux: 未找到amixer命令。请安装alsa-utils:\n"
                "sudo apt-get install alsa-utils  # Debian/Ubuntu\n"
                "sudo yum install alsa-utils      # CentOS/RHEL")
    
    def set_volume(self, volume_level: Optional[float] = None) -> Tuple[bool, float]:
        """
        设置系统主音量
        
        Args:
            volume_level: 音量级别(0.0-1.0), 如果未提供将使用默认值
        
        Returns:
            Tuple[bool, float]: 操作是否成功及设置的音量级别
            
        Raises:
            VolumeControlError: 音量控制失败时抛出
            InvalidVolumeLevelError: 无效音量级别时抛出
        """
        if not self.initialized:
            raise VolumeControlError("音量控制器未正确初始化")
        
        volume_level = volume_level if volume_level is not None else self.default_volume
        
        if not 0.0 <= volume_level <= 1.0:
            raise InvalidVolumeLevelError("音量级别必须在0.0到1.0之间")
        
        try:
            if CURRENT_PLATFORM == 'Windows':
                self._set_volume_windows(volume_level)
            elif CURRENT_PLATFORM == 'Darwin':
                self._set_volume_macos(volume_level)
            elif CURRENT_PLATFORM == 'Linux':
                self._set_volume_linux(volume_level)
            return True, volume_level
        except Exception as e:
            raise VolumeControlError(f"设置音量失败: {str(e)}")
    
    def _set_volume_windows(self, volume_level: float):
        """Windows平台音量设置"""
        self.volume_interface.SetMasterVolumeLevelScalar(volume_level, None)
    
    def _set_volume_macos(self, volume_level: float):
        """macOS平台音量设置"""
        volume_percent = int(volume_level * 100)
        script = f'set volume output volume {volume_percent}'
        
        try:
            subprocess.run(['osascript', '-e', script], check=True)
        except subprocess.CalledProcessError as e:
            raise VolumeControlError(f"AppleScript执行失败: {str(e)}")
    
    def _set_volume_linux(self, volume_level: float):
        """Linux平台音量设置"""
        volume_percent = int(volume_level * 100)
        
        try:
            subprocess.run(['amixer', '-D', 'pulse', 'sset', 'Master', f'{volume_percent}%'], 
                          check=True)
        except subprocess.CalledProcessError:
            subprocess.run(['amixer', 'sset', 'Master', f'{volume_percent}%'], 
                           check=True)
    
    def get_volume(self) -> Optional[float]:
        """
        获取当前系统音量
        
        Returns:
            float: 当前音量级别(0.0-1.0), 失败时返回None
        """
        if not self.initialized:
            print("音量控制器未正确初始化", file=sys.stderr)
            return None
        
        try:
            if CURRENT_PLATFORM == 'Windows':
                return self._get_volume_windows()
            elif CURRENT_PLATFORM == 'Darwin':
                return self._get_volume_macos()
            elif CURRENT_PLATFORM == 'Linux':
                return self._get_volume_linux()
        except Exception as e:
            print(f"获取音量失败: {str(e)}", file=sys.stderr)
            return None
    
    def _get_volume_windows(self) -> float:
        """Windows平台获取音量"""
        return self.volume_interface.GetMasterVolumeLevelScalar()
    
    def _get_volume_macos(self) -> float:
        """macOS平台获取音量"""
        script = 'get volume settings'
        result = subprocess.run(['osascript', '-e', script], 
                              check=True, capture_output=True, text=True)
        
        output = result.stdout.strip()
        volume_str = output.split(',')[0].split(':')[1]
        return float(volume_str) / 100
    
    def _get_volume_linux(self) -> float:
        """Linux平台获取音量"""
        try:
            result = subprocess.run(['amixer', '-D', 'pulse', 'get', 'Master'], 
                                  check=True, capture_output=True, text=True)
        except subprocess.CalledProcessError:
            result = subprocess.run(['amixer', 'get', 'Master'], 
                                   check=True, capture_output=True, text=True)
        
        lines = result.stdout.splitlines()
        for line in lines:
            if '[' in line and '%]' in line:
                percent_str = line.split('[')[1].split('%]')[0]
                try:
                    return float(percent_str) / 100
                except ValueError:
                    continue
        
        raise VolumeControlError("无法解析音量级别")

def print_usage():
    """打印使用说明"""
    print("""音量控制工具 - 使用说明

用法:
  python volume_control.py [volume_level]

参数:
  volume_level - 设置音量级别(0.0-1.0), 不提供参数则显示当前音量

示例:
  python volume_control.py 0.75   # 设置音量为75%
  python volume_control.py        # 显示当前音量
""")

def main():
    try:
        controller = VolumeControl(default_volume=0.5)
        
        # 在交互式环境中不处理命令行参数
        if not IS_INTERACTIVE and len(sys.argv) > 1:
            try:
                volume_level = float(sys.argv[1])
                success, level = controller.set_volume(volume_level)
                print(f"音量已设置为 {level*100:.0f}%")
            except ValueError:
                print("错误: 音量参数必须是数字", file=sys.stderr)
                print_usage()
                if not IS_INTERACTIVE:
                    sys.exit(1)
            except VolumeControlError as e:
                print(f"错误: {str(e)}", file=sys.stderr)
                if not IS_INTERACTIVE:
                    sys.exit(1)
        else:
            current_volume = controller.get_volume()
            if current_volume is not None:
                print(f"当前音量: {current_volume*100:.0f}%")
            else:
                print("无法获取当前音量", file=sys.stderr)
                if not IS_INTERACTIVE:
                    sys.exit(1)
    
    except UnsupportedPlatformError as e:
        print(f"错误: {str(e)}", file=sys.stderr)
        print("目前支持以下平台:", ", ".join(SUPPORTED_PLATFORMS), file=sys.stderr)
        if not IS_INTERACTIVE:
            sys.exit(1)
    except Exception as e:
        print(f"发生未预期的错误: {str(e)}", file=sys.stderr)
        if not IS_INTERACTIVE:
            sys.exit(1)

if __name__ == "__main__":
    main()


import os
import subprocess
import locale
from typing import Tuple

def remove_files_safely(path: str = "123") -> Tuple[int, str, str]:
    """
    安全地删除指定路径下的文件或目录。
    
    参数:
        path (str): 要删除的文件或目录的路径，默认为"123"。
        
    返回值:
        Tuple[int, str, str]: (退出代码, 标准输出, 标准错误)。
    """
    # 检查提供的路径是否存在
    if not os.path.exists(path):
        #pass
        return -1, "", f"指定的路径 {path} 不存在。"

    # 构建删除命令
    command = ["del", '/S', '/F', '/Q', path]
    
    def execute_command(command) -> Tuple[int, str, str]:
        """
        在 shell 中执行命令，并返回退出代码、标准输出和标准错误。
        """
        print(f"运行的命令: {' '.join(command)}")
        
        try:
            result = subprocess.run(
                command,
                capture_output=True,
                text=False,
                check=False,
                shell=True,
            )
            
            def decode_with_fallback(byte_data):
                """尝试 UTF-8 解码，如果失败则使用系统默认编码解码"""
                if not byte_data:
                    return ""
                try:
                    return byte_data.decode('utf-8').strip()
                except UnicodeDecodeError:
                    sys_encoding = locale.getpreferredencoding()
                    return byte_data.decode(sys_encoding, errors='replace').strip()

            stdout = decode_with_fallback(result.stdout)
            stderr = decode_with_fallback(result.stderr)

            print(f"命令输出 (stdout):\n{stdout}")
            print(f"命令错误 (stderr):\n{stderr}" if stderr else "命令错误 (stderr): 无")

            return result.returncode, stdout, stderr

        except Exception as e:
            print(f"执行命令时发生异常: {str(e)}")
            return -1, "", str(e)

    # 执行删除命令
    return execute_command(command)
def execute_command(command) -> Tuple[int, str, str]:
    """
    在 shell 中执行命令，并返回退出代码、标准输出和标准错误。
    """
    print(f"运行的命令: {' '.join(command)}")
    
    try:
        result = subprocess.run(
            command,
            capture_output=True,
            text=False,
            check=False,
            shell=True,
        )
        
        def decode_with_fallback(byte_data):
            """尝试 UTF-8 解码，如果失败则使用系统默认编码解码"""
            if not byte_data:
                return ""
            try:
                return byte_data.decode('utf-8').strip()
            except UnicodeDecodeError:
                sys_encoding = locale.getpreferredencoding()
                return byte_data.decode(sys_encoding, errors='replace').strip()

        stdout = decode_with_fallback(result.stdout)
        stderr = decode_with_fallback(result.stderr)

        print(f"命令输出 (stdout):\n{stdout}")
        print(f"命令错误 (stderr):\n{stderr}" if stderr else "命令错误 (stderr): 无")

        return result.returncode, stdout, stderr

    except Exception as e:
        print(f"执行命令时发生异常: {str(e)}")
        return -1, "", str(e)

# 示例调用
if __name__ == "__main__":
    ret_code, stdout, stderr = remove_files_safely("123")
    #print(f"操作结果: 退出代码={ret_code}, 标准输出={stdout}, 标准错误={stderr}")


import win32com.client
import pythoncom
import sys
import traceback

# Excel 常數 (對應到 VBA 常數)
xlContinuous = 1
xlThin = 2
xlExpression = 1
xlLeft = 1
xlRight = 2
xlTop = 3
xlBottom = 4
xlThemeColorDark1 = 1



def apply_border_conditional_formatting(sheet,target_range_str="A1:AA999", formula="=NOT(OR(A1=\"\",A1=0))"):
    """
    為指定的範圍加入條件式格式設定，並應用邊框樣式。
    """
    
    target_range = sheet.Range(target_range_str)
    try:
        from win32com.client import constants
        if target_range.MergeCells:
            print("目標範圍包含合併儲存格，請確認並重試。")
            return

        format_condition = target_range.FormatConditions.Add(Type=constants.xlExpression, Formula1=formula)
        format_condition.SetFirstPriority()
        print(f"已加入條件式格式設定規則 (邊框)，公式: {formula}。")

        for border_type in [xlLeft, xlRight, xlTop, xlBottom]:
            border = format_condition.Borders(border_type)
            border.LineStyle = xlContinuous
            border.Weight = xlThin
        print("已為條件式格式設定規則應用邊框。")
    except Exception as e:
        print(f"應用邊框條件式格式設定時發生錯誤: {e}")
        traceback.print_exc()




import win32com.client
import datetime
import os
import time
import subprocess
import pythoncom # 用于释放COM对象

def get_powerpoint_app():
    """获取或启动PowerPoint应用程序实例。"""
    try:
        powerpoint_app = win32com.client.Dispatch("PowerPoint.Application")
        powerpoint_app.Visible = True # 确保PowerPoint是可见的
        print("已连接到PowerPoint应用程序。")
        return powerpoint_app
    except Exception as e:
        print(f"无法启动或连接到PowerPoint应用程序: {e}")
        return None

def get_active_presentation(app):
    """获取当前活跃的演示文稿。"""
    if not app:
        return None
    
    try:
        if app.Presentations.Count == 0:
            print("没有打开的演示文稿。请打开一个PowerPoint文件。")
            return None
        return app.ActivePresentation
    except Exception as e:
        print(f"获取当前活跃演示文稿时发生错误: {e}")
        return None

def generate_timestamped_filepath(presentation, directory="D:\\"):
    """
    生成带时间戳的文件保存路径。
    
    Args:
        presentation: PowerPoint演示文稿对象。
        directory (str): 文件保存的目标目录。
        
    Returns:
        str: 完整的文件保存路径，如果无法生成则返回None。
    """
    if not presentation:
        return None
        
    try:
        timestamp = datetime.datetime.now().strftime("%Y%m%d_%H%M%S")
        original_filename = os.path.splitext(os.path.basename(presentation.FullName))[0]
        new_filename = f"{original_filename}_{timestamp}.pptx"
        new_filepath = os.path.join(directory, new_filename)
        print(f"生成的目标文件路径: {new_filepath}")
        return new_filepath
    except Exception as e:
        print(f"生成文件路径时发生错误: {e}")
        return None

def save_presentation(presentation, filepath):
    """
    保存演示文稿到指定路径。
    
    Args:
        presentation: PowerPoint演示文稿对象。
        filepath (str): 文件的完整保存路径。
        
    Returns:
        bool: 如果保存成功则返回True，否则返回False。
    """
    if not presentation or not filepath:
        return False
        
    try:
        presentation.SaveAs(filepath)
        print(f"演示文稿已成功保存到: {filepath}")
        return True
    except Exception as e:
        print(f"保存演示文稿时发生错误: {e}")
        return False

def copy_file_to_clipboard(filepath):
    """
    将指定文件复制到剪贴板。
    使用PowerShell命令模拟Windows的文件复制操作。
    
    Args:
        filepath (str): 要复制的文件的完整路径。
        
    Returns:
        bool: 如果复制成功则返回True，否则返回False。
    """
    if not filepath or not os.path.exists(filepath):
        print(f"文件 '{filepath}' 不存在或路径无效，无法复制到剪贴板。")
        return False

    try:
        powershell_command = f'Set-Clipboard -LiteralPath "{filepath}"'
        # CREATE_NO_WINDOW 隐藏PowerShell窗口
        subprocess.run(["powershell", "-Command", powershell_command], check=True, creationflags=subprocess.CREATE_NO_WINDOW)
        print("文件已成功复制到剪贴板。")
        return True
    except subprocess.CalledProcessError as e:
        print(f"PowerShell命令执行失败: {e}")
        print(f"错误输出: {e.stderr}")
        return False
    except Exception as e:
        print(f"复制文件到剪贴板时发生错误: {e}")
        return False

def cleanup_powerpoint(app):
    """
    清理并关闭PowerPoint应用程序实例。
    
    Args:
        app: PowerPoint应用程序对象。
    """
    if app:
        try:
            app.Quit()
            # 释放COM对象，防止资源泄露
            pythoncom.CoUninitialize() # 可能需要根据具体情况调整，通常在Dispatch后不需要手动CoUninitialize
            print("PowerPoint 应用程序已退出。")
        except Exception as e:
            print(f"关闭PowerPoint应用程序时发生错误: {e}")

def main_process():
    """协调整个保存、复制和退出的主流程。"""
    powerpoint_app = None # 初始化为None

    try:
        powerpoint_app = get_powerpoint_app()
        if not powerpoint_app:
            return

        current_presentation = get_active_presentation(powerpoint_app)
        if not current_presentation:
            return

        new_filepath = generate_timestamped_filepath(current_presentation, "D:\\")
        if not new_filepath:
            return

        if not save_presentation(current_presentation, new_filepath):
            return

        if not copy_file_to_clipboard(new_filepath):
            print("文件复制到剪贴板操作未能成功完成。")
            
        print("程序将在5秒后退出...")
        time.sleep(5)

    except Exception as e:
        print(f"主程序执行过程中发生意外错误: {e}")
    finally:
        cleanup_powerpoint(powerpoint_app)
        
if __name__ == "__main__":
    main_process()


bbb = r'https://d.docs.live.net/9122e41a29eea899/sb_yufengguang/xls/%E6%A8%A1%E6%9D%BF-%E9%9D%9E%E5%B8%B8%E9%87%8D%E8%A6%81/%E5%89%AF%E6%9C%AC%E5%AE%B6%E5%BA%AD%E8%B4%A6%E7%B0%BF%E6%9B%B4%E6%96%B0%E4%BA%8E2025%E5%B9%B42%E6%9C%887%E6%97%A5.xlsx'
aaa = '副本家庭账簿更新于2025年2月7日_20250507201736308190'
bbb

import f_excel.d单函数.open_or_add_process_excel_with_r1c1 as op


op.open_or_add_process_excel_with_r1c1_0(bbb)

from time import *
sleep(10)


import streamlit as st
import pandas as pd
import numpy as np

st.title("数据展示示例")

df = pd.DataFrame(
    np.random.randn(10, 5),
    columns=('col %d' % i for i in range(5))
)

st.subheader("使用 `st.dataframe()` 展示数据帧 (交互式)")
st.dataframe(df)

st.subheader("使用 `st.table()` 展示静态表格")
st.table(df.head())

st.subheader("使用 `st.metric()` 展示关键指标")
st.metric(label="温度", value="28°C", delta="1.2°C")
st.metric(label="湿度", value="65%", delta="-2%")



aaa = "中美为何无法达成协议 xls workbook_20250509151812190427"

bbb = r'https://d.docs.live.net/9122e41a29eea899/sb_yufengguang/xls/%E4%B8%AD%E7%BE%8E%E4%B8%BA%E4%BD%95%E6%97%A0%E6%B3%95%E8%BE%BE%E6%88%90%E5%8D%8F%E8%AE%AE%20xls%20workbook_20250509151812190427.xlsx'



import f_excel.d单函数.open_or_add_process_excel_with_r1c1 as op


op.open_or_add_process_excel_with_r1c1_0(bbb)

from time import *
sleep(10)


bbb = r'https://d.docs.live.net/380c326fa2fd9c70/%E7%A7%81%E4%BA%BA%E6%96%87%E4%BB%B6%EF%BC%8Cdengchunying1988/%E6%A1%8C%E9%9D%A2%201/%E5%AD%A6%E4%B9%A0/%E5%AE%B6%E5%BA%AD%E8%B4%A6%E7%B0%BF%E6%9B%B4%E6%96%B0%E4%BA%8E2025%E5%B9%B43%E6%9C%8812%E6%97%A5.xlsx'
aaa = '家庭账簿更新于2025年3月12日'

import frankyu.xlsx.initialize_excel as xl
app,b,c = xl.initialize_excel()

 
ccc = ""
for i in app.Workbooks:
    #print(i.Name)
    if aaa in i.Name :
        ccc = aaa
        #print(ccc)
        print(aaa,"已经打开,程序退出")
        break
if ccc == "":
    app.Workbooks.Open(bbb)
    print(aaa,"已经打开")
    
    
    
    
import frankyu.frankyu as fr
fr.daoJiShi_t2(5)



# 导入win32com客户端模块，用于与Excel进行交互
import win32com.client as win32

# 导入os模块，用于文件系统操作（如路径处理和文件删除）
import os

# 从datetime模块导入datetime类，用于生成时间戳
from datetime import datetime

# 导入time模块，用于程序延迟操作
import time

# -------------------------------------------------------------------
# 函数：初始化并配置Excel应用程序
def initialize_excel_application() -> win32.Dispatch:
    """初始化并配置Excel应用程序
    
    Returns:
        win32.Dispatch: Excel应用程序对象
    """
    # 创建Excel应用程序对象
    excel_app = win32.gencache.EnsureDispatch('Excel.Application')
    
    # 设置Excel前台运行（可视化）
    excel_app.Visible = True          # 显示Excel界面
    
    # 禁用所有警告弹窗（如覆盖文件、公式错误等）
    excel_app.DisplayAlerts = False   # 关闭警告提示
    
    # 返回配置好的Excel应用程序对象
    return excel_app

# -------------------------------------------------------------------
# 函数：创建新的Excel工作簿
def create_new_workbook(excel_app: win32.Dispatch) -> win32.Dispatch:
    """创建新的Excel工作簿
    
    Args:
        excel_app (win32.Dispatch): Excel应用程序对象
    
    Returns:
        win32.Dispatch: 新建的工作簿对象
    """
    # 调用Excel的Workbooks.Add方法创建新工作簿
    return excel_app.Workbooks.Add()

# -------------------------------------------------------------------
# 函数：复制指定单元格区域到目标位置
def copy_cell_range(
    worksheet: win32.Dispatch, 
    source_range: str, 
    destination_cell: str
):
    """复制指定单元格区域到目标位置
    
    Args:
        worksheet (win32.Dispatch): 目标工作表对象
        source_range (str): 需要复制的源区域（如"A1:E2"）
        destination_cell (str): 粘贴的目标单元格（如"A5"）
    """
    # 获取源区域对象
    source = worksheet.Range(source_range)
    
    # 获取目标单元格对象
    destination = worksheet.Range(destination_cell)
    
    # 执行复制操作（将源区域内容复制到目标位置）
    source.Copy(Destination=destination)
    
    # 输出复制操作完成信息
    print(f"已复制区域 {source_range} 到 {destination_cell}")

# -------------------------------------------------------------------
# 函数：配置工作表内容（重命名、填充数据、复制内容）
def configure_worksheet_content(workbook: win32.Dispatch):
    """配置工作表内容：重命名、填充数据、复制内容
    
    Args:
        workbook (win32.Dispatch): 需要配置的工作簿对象
    """
    # 获取工作簿的第一个工作表
    main_sheet = workbook.Sheets(1)
    
    # 重命名工作表为"DataSheet"
    main_sheet.Name = 'DataSheet'    # 数据表
    
    # 遍历行索引（1到2行）
    for row in range(1, 3):
        # 遍历列索引（1到5列）
        for col in range(1, 6):
            # 生成单元格值格式（如Row1Col1）
            cell_value = f"Row{row}Col{col}"
            
            # 将生成的值写入当前单元格
            main_sheet.Cells(row, col).Value = cell_value
    
    # 调用独立函数执行复制操作
    copy_cell_range(
        worksheet=main_sheet,
        source_range="A1:E2",
        destination_cell="A5"
    )
    
    # 添加新工作表（用于汇总）
    new_sheet = workbook.Worksheets.Add()
    
    # 重命名新工作表为"SummarySheet"
    new_sheet.Name = 'SummarySheet'  # 汇总表

# -------------------------------------------------------------------
# 函数：生成带时间戳的文件名
def generate_timestamped_filename(base_dir: str = "T:\\") -> str:
    """生成带时间戳的唯一文件名
    
    Args:
        base_dir (str, optional): 文件保存根目录. 默认为 "T:\\"
    
    Returns:
        str: 完整文件路径（包含时间戳）
    """
    # 获取当前时间并格式化为"YYYYMMDD_HHMMSS"
    timestamp = datetime.now().strftime("%Y%m%d_%H%M%S")
    
    # 生成文件名格式（如Report_20231005_143025.xlsx）
    filename = f"Report_{timestamp}.xlsx"
    
    # 合并根目录和文件名，返回完整路径
    return os.path.join(base_dir, filename)

# -------------------------------------------------------------------
# 函数：保存工作簿到指定路径
def save_workbook(workbook: win32.Dispatch, output_path: str):
    """保存工作簿到指定路径
    
    Args:
        workbook (win32.Dispatch): 需要保存的工作簿对象
        output_path (str): 文件保存的完整路径
    """
    # 检查文件是否存在
    if os.path.exists(output_path):
        # 强制删除旧文件（如果存在）
        os.remove(output_path)  
        
    # 保存工作簿到指定路径
    workbook.SaveAs(output_path)
    
    # 输出文件保存路径信息
    print(f"文件已保存至: {output_path}")

# -------------------------------------------------------------------
# 函数：延后关闭Excel应用程序
def close_excel_application(excel_app: win32.Dispatch, delay: int = 30):
    """延后关闭Excel应用程序
    
    Args:
        excel_app (win32.Dispatch): Excel应用程序对象
        delay (int, optional): 关闭前的等待时间（秒）. 默认30秒
    """
    # 输出关闭倒计时信息
    print(f"将在{delay}秒后关闭Excel...")
    
    # 暂停程序执行，等待用户查看结果
    time.sleep(delay)
    
    # 关闭Excel应用程序
    excel_app.Quit()
    
    # 输出关闭完成信息
    print("Excel已关闭")

# -------------------------------------------------------------------
# 主函数：程序入口
def main():
    """程序主函数：协调各功能模块"""
    # 初始化Excel应用程序对象（初始值为None）
    excel_app = None
    
    # 初始化工作簿对象（初始值为None）
    workbook = None
    
    try:
        # 1. 初始化Excel（前台运行）
        # 调用初始化函数获取Excel对象
        excel_app = initialize_excel_application()
        
        # 2. 创建新工作簿
        # 调用创建函数获取新工作簿对象
        workbook = create_new_workbook(excel_app)
        
        # 3. 配置工作表内容
        # 调用配置函数设置工作表数据
        configure_worksheet_content(workbook)
        
        # 4. 生成带时间戳的文件路径
        # 调用生成文件名函数获取保存路径
        output_path = generate_timestamped_filename()
        
        # 5. 保存工作簿
        # 调用保存函数将数据保存到指定路径
        save_workbook(workbook, output_path)

    except Exception as e:
        # 捕获异常并输出错误信息
        print(f"错误：{str(e)}")
    finally:
        # 安全关闭Excel（无论是否发生异常）
        if excel_app:
            # 调用关闭函数终止Excel进程
            close_excel_application(excel_app)

if __name__ == "__main__":
    # 程序入口，仅当直接运行脚本时执行
    main()


"""
Windows.edb 是 Windows Search 索引文件，主要用于提供内容索引、属性缓存以及文件、电子邮件和其他内容的搜索结果。
由于 Windows 系统默认会对文件进行索引以加快搜索速度，所有与索引有关的数据都存储在这个 edb 文件中。
随着使用时间的增长，Windows.edb 文件可能会变得非常大，占用大量磁盘空间。

操作步骤：
1. 停止 Windows Search 服务以便释放文件锁定。
2. 删除 Windows.edb 文件，释放磁盘空间。
3. 根据需要重新启动 Windows Search 服务并配置其启动类型。
4. 如果希望完全禁用生成 Windows.edb 文件的功能，需要禁用 Windows Search 服务。

注意事项：
- 在操作前，请确保您对系统的操作有足够的权限（以管理员身份运行）。
- 删除 Windows.edb 文件后，若重新启用 Windows Search 服务，该文件会自动重新生成。
"""

# 定义命令字符串，用于控制 Windows Search 服务的启动和停止
aaa = "net stop wsearch"  # 停止 Windows Search 服务的命令
bbb = r"sc config wsearch start= disabled"  # 禁用 Windows Search 服务的自动启动命令
ccc = r"net start wsearch"  # 启动 Windows Search 服务的命令
eee = "sc config wsearch start= delayed-auto"  # 配置 Windows Search 服务的启动类型为延迟自动

# 循环执行停止和禁用服务的命令
for i in [aaa, bbb]:
    import os
    import time
    
    # 使用 os.system 执行系统命令
    # os.system(i)  # 停止服务或禁用自动启动（注释掉以防止误执行）
    
    # 为了安全，延迟 10 秒，确保命令执行完成
    # time.sleep(10)

# 循环执行启动和配置服务的命令
for i in [ccc, eee]:
    import os
    import time
    
    os.system(i)  # 启动服务或配置启动类型
    time.sleep(10)  # 延迟 10 秒，确保命令执行完成


import streamlit as st
import time

st.title("状态和进度示例")

# Spinner
with st.spinner('正在执行耗时操作...'):
    time.sleep(3)
st.success('操作完成！')

# 进度条
st.subheader("进度条")
progress_text = "操作进行中。请稍候。"
my_bar = st.progress(0, text=progress_text)

for percent_complete in range(100):
    time.sleep(0.05)
    my_bar.progress(percent_complete + 1, text=progress_text)
st.success('所有任务已完成！')

# 气球
if st.button("庆祝一下！"):
    st.balloons()

# 下雪
if st.button("下雪！"):
    st.snow()

# Toast
if st.button("显示通知"):
    st.toast('这是一个短暂的通知！')


import win32com.client
import pythoncom
import sys
import traceback

# Excel 常數 (對應到 VBA 常數)
xlContinuous = 1
xlThin = 2
xlExpression = 1
xlLeft = 1
xlRight = 2
xlTop = 3
xlBottom = 4
xlThemeColorDark1 = 1


def set_cell_value_and_select(sheet, value_cell="A1", value_to_set=1, select_cell="A2"):
    """
    設定指定儲存格的值，並選取另一個儲存格。
    """
    
    try:
        sheet.Range("A3").Select()
        sheet.Range(value_cell).Value = value_to_set
        print(f"已將儲存格 {value_cell} 的值設定為 {value_to_set}。")
        if select_cell:
            
            sheet.Range(select_cell).Select()
            
        print(f"已選取儲存格 {select_cell}。")
    except Exception as e:
        print(f"設定儲存格值或選取儲存格時發生錯誤: {e}")
        traceback.print_exc()



import streamlit as st

st.title("布局示例")

# 侧边栏
st.sidebar.header("侧边栏")
st.sidebar.text_input("侧边栏输入框", "你好")

# 多列布局
st.header("多列布局")
col1, col2, col3 = st.columns(3)
with col1:
    st.write("这是第一列")
    st.button("按钮1")
with col2:
    st.write("这是第二列")
    st.button("按钮2")
with col3:
    st.write("这是第三列")
    st.button("按钮3")

# 选项卡布局
st.header("选项卡布局")
tab1, tab2, tab3 = st.tabs(["介绍", "数据", "图表"])
with tab1:
    st.write("这是介绍页面的内容。")
with tab2:
    st.write("这里可以展示数据表格。")
with tab3:
    st.write("这里可以展示数据图表。")

# 容器
st.header("容器")
with st.container(border=True):
    st.write("这个容器包含了一段文本和一个滑块。")
    st.slider("容器内的滑块", 0, 10)

# 可折叠区域
st.header("可折叠区域")
with st.expander("点击展开更多信息"):
    st.write("这里是一些隐藏起来的信息。")
    st.image("https://static.streamlit.io/examples/cat.jpg", width=20)


aaa = r'''

python 程序打包


# 使用 venv 创建虚拟环境
python -m venv myenv
# 激活虚拟环境
# Windows
myenv\Scripts\activate
# macOS/Linux
source myenv/bin/activate
 
# 安装依赖
pip install -r requirements.txt


pip install pyinstaller
pyinstaller your_script.py


https://www.baidu.com/s?ie=utf-8&f=8&rsv_bp=0&rsv_idx=1&tn=baidu&wd=python%20%E7%A8%8B%E5%BA%8F%E6%89%93%E5%8C%85&rsv_pq=ce9526f2000081a6&rsv_t=bcc9h0B0Y3Bzy7FKFBir0EgRB2Y8BgOLZzf6s70mFcAitgfO02kIoxl9gWw&rqlang=cn&rsv_enter=1&rsv_sug3=4&rsv_sug1=3&rsv_sug7=100&rsv_sug2=0&inputT=1394&rsv_sug4=
'''

print(aaa)


import win32com.client as wc

# 1. 启动 PowerPoint 应用程序并使其可见
def create_ppt_application():
    """
    启动 PowerPoint 应用程序并使其可见。
    """
    try:
        # 获取 PowerPoint 应用程序对象
        ppt_app = wc.Dispatch("PowerPoint.Application")
        # 使 PowerPoint 应用程序可见 (True 可见, False 不可见)
        ppt_app.Visible = True
        print("PowerPoint 应用程序已启动并可见。")
        return ppt_app
    except Exception as e:
        print(f"启动 PowerPoint 应用程序失败: {e}")
        return None

# 2. 创建一个新的演示文稿
def create_new_presentation(ppt_app):
    """
    在给定的 PowerPoint 应用程序中创建一个新的演示文稿。
    """
    if not ppt_app:
        print("PowerPoint 应用程序未启动。")
        return None
    try:
        # 添加一个新的演示文稿
        presentation = ppt_app.Presentations.Add()
        print("已创建一个新的演示文稿。")
        return presentation
    except Exception as e:
        print(f"创建新演示文稿失败: {e}")
        return None

# 3. 打开一个已存在的演示文稿
def open_existing_presentation(ppt_app, file_path):
    """
    在给定的 PowerPoint 应用程序中打开一个已存在的演示文稿。
    :param ppt_app: PowerPoint 应用程序对象
    :param file_path: 演示文稿的完整路径
    """
    if not ppt_app:
        print("PowerPoint 应用程序未启动。")
        return None
    try:
        # 打开一个已存在的演示文稿 (第二个参数是 ReadOnly, True 表示只读)
        presentation = ppt_app.Presentations.Open(file_path, False, False, False)
        print(f"已打开演示文稿: {file_path}")
        return presentation
    except Exception as e:
        print(f"打开演示文稿失败: {e}")
        return None

# 4. 保存演示文稿
def save_presentation(presentation, save_path):
    """
    保存演示文稿到指定路径。
    :param presentation: 演示文稿对象
    :param save_path: 保存的完整路径和文件名
    """
    if not presentation:
        print("演示文稿对象无效。")
        return
    try:
        # 保存演示文稿
        presentation.SaveAs(save_path)
        print(f"演示文稿已保存到: {save_path}")
    except Exception as e:
        print(f"保存演示文稿失败: {e}")

# 5. 关闭演示文稿
def close_presentation(presentation):
    """
    关闭演示文稿。
    :param presentation: 演示文稿对象
    """
    if not presentation:
        print("演示文稿对象无效。")
        return
    try:
        # 关闭演示文稿
        presentation.Close()
        print("演示文稿已关闭。")
    except Exception as e:
        print(f"关闭演示文稿失败: {e}")

# 6. 退出 PowerPoint 应用程序
def quit_ppt_application(ppt_app):
    """
    退出 PowerPoint 应用程序。
    :param ppt_app: PowerPoint 应用程序对象
    """
    if not ppt_app:
        print("PowerPoint 应用程序未启动。")
        return
    try:
        # 退出 PowerPoint 应用程序
        ppt_app.Quit()
        print("PowerPoint 应用程序已退出。")
    except Exception as e:
        print(f"退出 PowerPoint 应用程序失败: {e}")

# 7. 获取当前活动演示文稿
def get_active_presentation(ppt_app):
    """
    获取当前活动演示文稿。
    :param ppt_app: PowerPoint 应用程序对象
    """
    if not ppt_app:
        print("PowerPoint 应用程序未启动。")
        return None
    try:
        # 获取当前活动演示文稿
        active_presentation = ppt_app.ActivePresentation
        if active_presentation:
            print(f"当前活动演示文稿的名称是: {active_presentation.Name}")
            return active_presentation
        else:
            print("当前没有活动演示文稿。")
            return None
    except Exception as e:
        print(f"获取当前活动演示文稿失败: {e}")
        return None

# 8. 添加一张新的幻灯片
def add_new_slide(presentation, layout_index=1):
    """
    在演示文稿中添加一张新的幻灯片。
    :param presentation: 演示文稿对象
    :param layout_index: 幻灯片布局的索引 (例如: 1 表示标题幻灯片, 2 表示标题和内容幻灯片)
                         详见 MSO.MsoSlideLayout 枚举
                         https://learn.microsoft.com/zh-cn/office/vba/api/powerpoint.ppslidelayout
    """
    if not presentation:
        print("演示文稿对象无效。")
        return None
    try:
        # 获取幻灯片集合
        slides = presentation.Slides
        # 添加一张新的幻灯片
        # MsoSlideLayout.ppLayoutTitle (1) - 标题幻灯片
        # MsoSlideLayout.ppLayoutTitleAndContent (2) - 标题和内容幻灯片
        # MsoSlideLayout.ppLayoutBlank (12) - 空白幻灯片
        slide = slides.Add(slides.Count + 1, layout_index)
        print(f"已添加一张新的幻灯片，布局索引为: {layout_index}")
        return slide
    except Exception as e:
        print(f"添加新幻灯片失败: {e}")
        return None

# 9. 获取指定索引的幻灯片
def get_slide_by_index(presentation, index):
    """
    获取演示文稿中指定索引的幻灯片。
    :param presentation: 演示文稿对象
    :param index: 幻灯片索引 (从 1 开始)
    """
    if not presentation:
        print("演示文稿对象无效。")
        return None
    try:
        # 获取指定索引的幻灯片
        slide = presentation.Slides(index)
        print(f"已获取索引为 {index} 的幻灯片。")
        return slide
    except Exception as e:
        print(f"获取索引为 {index} 的幻灯片失败: {e}")
        return None

# 10. 删除指定索引的幻灯片
def delete_slide_by_index(presentation, index):
    """
    删除演示文稿中指定索引的幻灯片。
    :param presentation: 演示文稿对象
    :param index: 幻灯片索引 (从 1 开始)
    """
    if not presentation:
        print("演示文稿对象无效。")
        return
    try:
        # 删除指定索引的幻灯片
        presentation.Slides(index).Delete()
        print(f"已删除索引为 {index} 的幻灯片。")
    except Exception as e:
        print(f"删除索引为 {index} 的幻灯片失败: {e}")

# 11. 在幻灯片中添加文本框并设置文本
def add_text_box_to_slide(slide, text, left, top, width, height):
    """
    在幻灯片中添加一个文本框并设置其文本。
    :param slide: 幻灯片对象
    :param text: 要设置的文本内容
    :param left: 文本框左上角的 X 坐标
    :param top: 文本框左上角的 Y 坐标
    :param width: 文本框宽度
    :param height: 文本框高度
    """
    if not slide:
        print("幻灯片对象无效。")
        return None
    try:
        # 添加一个文本框 (MsoShapeType.msoTextBox)
        # Left, Top, Width, Height 都是以磅为单位
        shape = slide.Shapes.AddTextbox(1, left, top, width, height) # 1 代表 msoTextOrientationHorizontal
        # 获取文本框的文本帧
        text_frame = shape.TextFrame
        # 获取文本帧的文本范围
        text_range = text_frame.TextRange
        # 设置文本内容
        text_range.Text = text
        print(f"已在幻灯片中添加文本框并设置文本: '{text}'")
        return shape
    except Exception as e:
        print(f"添加文本框失败: {e}")
        return None

# 12. 设置文本框字体样式
def set_text_box_font(shape, font_name="微软雅黑", font_size=24, bold=False, italic=False, underline=False):
    """
    设置文本框的字体样式。
    :param shape: 形状对象 (文本框)
    :param font_name: 字体名称
    :param font_size: 字体大小
    :param bold: 是否加粗
    :param italic: 是否斜体
    :param underline: 是否下划线
    """
    if not shape or not hasattr(shape, 'TextFrame'):
        print("形状对象无效或不是文本框。")
        return
    try:
        font = shape.TextFrame.TextRange.Font
        font.Name = font_name
        font.Size = font_size
        font.Bold = bold
        font.Italic = italic
        font.Underline = underline
        print(f"已设置文本框字体: 名称='{font_name}', 大小={font_size}, 加粗={bold}, 斜体={italic}, 下划线={underline}")
    except Exception as e:
        print(f"设置文本框字体失败: {e}")

# 13. 在幻灯片中插入图片
def insert_picture_to_slide(slide, picture_path, left, top, width=-1, height=-1):
    """
    在幻灯片中插入图片。
    :param slide: 幻灯片对象
    :param picture_path: 图片文件的完整路径
    :param left: 图片左上角的 X 坐标
    :param top: 图片左上角的 Y 坐标
    :param width: 图片宽度 (可选, -1 表示自动调整)
    :param height: 图片高度 (可选, -1 表示自动调整)
    """
    if not slide:
        print("幻灯片对象无效。")
        return None
    try:
        # 插入图片 (MsoPictureCmd.msoPictureNormal 插入普通图片)
        # Left, Top, Width, Height 都是以磅为单位
        # -1 表示保持原始尺寸比例
        shape = slide.Shapes.AddPicture(picture_path, False, True, left, top, width, height)
        print(f"已在幻灯片中插入图片: {picture_path}")
        return shape
    except Exception as e:
        print(f"插入图片失败: {e}")
        return None

# 14. 获取幻灯片中所有形状的数量
def get_slide_shapes_count(slide):
    """
    获取幻灯片中所有形状的数量。
    :param slide: 幻灯片对象
    """
    if not slide:
        print("幻灯片对象无效。")
        return 0
    try:
        count = slide.Shapes.Count
        print(f"幻灯片中共有 {count} 个形状。")
        return count
    except Exception as e:
        print(f"获取形状数量失败: {e}")
        return 0

# 15. 按名称查找幻灯片中的形状
def find_shape_by_name(slide, shape_name):
    """
    按名称查找幻灯片中的形状。
    :param slide: 幻灯片对象
    :param shape_name: 形状的名称
    """
    if not slide:
        print("幻灯片对象无效。")
        return None
    try:
        # 遍历幻灯片中的所有形状
        for shape in slide.Shapes:
            if shape.Name == shape_name:
                print(f"已找到名称为 '{shape_name}' 的形状。")
                return shape
        print(f"未找到名称为 '{shape_name}' 的形状。")
        return None
    except Exception as e:
        print(f"查找形状失败: {e}")
        return None

# 16. 设置幻灯片背景颜色 (需要使用RGB值)
def set_slide_background_color(slide, r, g, b):
    """
    设置幻灯片背景颜色。
    :param slide: 幻灯片对象
    :param r: 红色分量 (0-255)
    :param g: 绿色分量 (0-255)
    :param b: 蓝色分量 (0-255)
    """
    if not slide:
        print("幻灯片对象无效。")
        return
    try:
        # 获取背景填充对象
        background_fill = slide.Background.Fill
        # 设置填充类型为纯色填充 (msoFillSolid)
        background_fill.PresetGradient(1, 1) # 随便选择一个渐变类型，然后改为纯色
        background_fill.Solid()
        # 获取背景的 ForeColor 对象
        background_color = background_fill.ForeColor
        # 设置 RGB 颜色
        background_color.RGB = r + (g << 8) + (b << 16)
        print(f"已设置幻灯片背景颜色为 RGB({r}, {g}, {b})。")
    except Exception as e:
        print(f"设置幻灯片背景颜色失败: {e}")

# 17. 获取幻灯片中的所有文本框（形状类型为 msoTextBox）
def get_all_text_boxes_on_slide(slide):
    """
    获取幻灯片中所有文本框。
    :param slide: 幻灯片对象
    :return: 文本框形状列表
    """
    if not slide:
        print("幻灯片对象无效。")
        return []
    text_boxes = []
    try:
        # 遍历幻灯片中的所有形状
        for shape in slide.Shapes:
            # 判断形状类型是否为文本框 (msoShapeType.msoTextBox = 17)
            if shape.Type == 17:  # MsoShapeType.msoTextBox
                text_boxes.append(shape)
                print(f"找到文本框: {shape.Name}, 文本: {shape.TextFrame.TextRange.Text.strip()}")
        return text_boxes
    except Exception as e:
        print(f"获取所有文本框失败: {e}")
        return []

# 18. 修改幻灯片标题
def set_slide_title(slide, title_text):
    """
    修改幻灯片标题。
    :param slide: 幻灯片对象
    :param title_text: 新的标题文本
    """
    if not slide:
        print("幻灯片对象无效。")
        return
    try:
        # 标题形状通常是幻灯片中的第一个形状，或者通过 PlaceholderType 识别
        for shape in slide.Shapes:
            if shape.HasTextFrame and shape.TextFrame.HasText:
                # 检查形状是否是标题占位符
                if shape.Type == 14: # msoPlaceholder
                    if shape.PlaceholderFormat.Type == 1: # ppPlaceholderTitle
                        shape.TextFrame.TextRange.Text = title_text
                        print(f"幻灯片标题已修改为: '{title_text}'")
                        return
        print("未找到幻灯片标题形状。")
    except Exception as e:
        print(f"修改幻灯片标题失败: {e}")

# 19. 插入表格到幻灯片
def insert_table_to_slide(slide, rows, columns, left, top, width, height):
    """
    在幻灯片中插入表格。
    :param slide: 幻灯片对象
    :param rows: 行数
    :param columns: 列数
    :param left: 表格左上角的 X 坐标
    :param top: 表格左上角的 Y 坐标
    :param width: 表格宽度
    :param height: 表格高度
    """
    if not slide:
        print("幻灯片对象无效。")
        return None
    try:
        # 插入表格
        shape = slide.Shapes.AddTable(rows, columns, left, top, width, height)
        print(f"已在幻灯片中插入一个 {rows} 行 {columns} 列的表格。")
        return shape
    except Exception as e:
        print(f"插入表格失败: {e}")
        return None

# 20. 设置表格单元格文本
def set_table_cell_text(table_shape, row, column, text):
    """
    设置表格指定单元格的文本。
    :param table_shape: 表格形状对象
    :param row: 行索引 (从 1 开始)
    :param column: 列索引 (从 1 开始)
    :param text: 单元格文本
    """
    if not table_shape or not hasattr(table_shape, 'Table'):
        print("表格形状对象无效。")
        return
    try:
        table = table_shape.Table
        cell = table.Cell(row, column)
        cell.Shape.TextFrame.TextRange.Text = text
        print(f"表格 ({row},{column}) 单元格文本已设置为: '{text}'")
    except Exception as e:
        print(f"设置表格单元格文本失败: {e}")

# 21. 插入图表到幻灯片
def insert_chart_to_slide(slide, chart_type, left, top, width, height):
    """
    在幻灯片中插入图表。
    :param slide: 幻灯片对象
    :param chart_type: 图表类型 (例如: -1 代表默认簇状柱形图，更多类型详见 MSO.XlChartType 枚举)
                        https://learn.microsoft.com/zh-cn/office/vba/api/excel.xlcharttype
    :param left: 图表左上角的 X 坐标
    :param top: 图表左上角的 Y 坐标
    :param width: 图表宽度
    :param height: 图表高度
    """
    if not slide:
        print("幻灯片对象无效。")
        return None
    try:
        # 插入图表
        # chart_type = -1 # 默认簇状柱形图 (xlColumnClustered)
        # 例如: wc.constants.xlPie (饼图), wc.constants.xlLine (折线图)
        shape = slide.Shapes.AddChart(chart_type, left, top, width, height)
        print(f"已在幻灯片中插入图表，类型为: {chart_type}")
        return shape
    except Exception as e:
        print(f"插入图表失败: {e}")
        return None

# 22. 获取图表数据表并修改数据
def update_chart_data(chart_shape, series_index, new_values):
    """
    更新图表的数据。
    :param chart_shape: 图表形状对象
    :param series_index: 系列索引 (从 1 开始)
    :param new_values: 新的数据列表 (例如: [10, 20, 30])
    """
    if not chart_shape or not hasattr(chart_shape, 'Chart'):
        print("图表形状对象无效。")
        return
    try:
        chart = chart_shape.Chart
        # 获取图表数据
        chart_data = chart.ChartData
        # 激活数据表
        chart_data.Activate()
        # 获取工作簿
        workbook = chart_data.Workbook
        # 获取第一个工作表
        worksheet = workbook.Sheets(1)

        # 清除现有数据 (可选，取决于更新策略)
        # worksheet.Cells.ClearContents()

        # 写入新数据
        for i, value in enumerate(new_values):
            # 假设数据从 B2 开始 (A列通常是类别标签)
            worksheet.Cells(i + 2, series_index + 1).Value = value

        # 重新应用数据源范围 (如果数据量变化需要重新设置)
        # 例如: chart.SetSourceData("Sheet1!$A$1:$B$4")

        # 保存并关闭工作簿
        workbook.Close()
        print(f"图表系列 {series_index} 数据已更新。")
    except Exception as e:
        print(f"更新图表数据失败: {e}")
        # 尝试关闭工作簿以避免悬挂
        if 'workbook' in locals() and workbook:
            workbook.Close(False) # False 表示不保存更改

# 23. 修改图表标题
def set_chart_title(chart_shape, title_text):
    """
    设置图表标题。
    :param chart_shape: 图表形状对象
    :param title_text: 新的图表标题文本
    """
    if not chart_shape or not hasattr(chart_shape, 'Chart'):
        print("图表形状对象无效。")
        return
    try:
        chart = chart_shape.Chart
        chart.HasTitle = True # 确保有标题
        chart.ChartTitle.Text = title_text
        print(f"图表标题已设置为: '{title_text}'")
    except Exception as e:
        print(f"设置图表标题失败: {e}")

# 24. 导出幻灯片为图片
def export_slide_as_image(slide, output_path, image_format="PNG"):
    """
    将幻灯片导出为图片。
    :param slide: 幻灯片对象
    :param output_path: 输出图片文件的完整路径 (例如: "C:\\temp\\slide1.png")
    :param image_format: 图片格式 ("PNG", "JPG", "BMP", "GIF")
    """
    if not slide:
        print("幻灯片对象无效。")
        return
    try:
        # 导出幻灯片
        # image_format 对应 MsoExportType 枚举，但这里通常直接用字符串
        slide.Export(output_path, image_format)
        print(f"幻灯片已导出为图片: {output_path}")
    except Exception as e:
        print(f"导出幻灯片为图片失败: {e}")

# 25. 设置幻灯片切换效果
def set_slide_transition(slide, transition_type=3, speed=1):
    """
    设置幻灯片切换效果。
    :param slide: 幻灯片对象
    :param transition_type: 切换效果类型 (例如: 3 为淡入淡出，更多类型详见 MsoAnimateType 枚举)
                            https://learn.microsoft.com/zh-cn/office/vba/api/powerpoint.ppeffect
    :param speed: 切换速度 (1: 快, 2: 中, 3: 慢)
    """
    if not slide:
        print("幻灯片对象无效。")
        return
    try:
        entry_effect = slide.SlideShowTransition
        entry_effect.EntryEffect = transition_type
        entry_effect.Speed = speed # 1-3
        print(f"幻灯片切换效果已设置为: 类型={transition_type}, 速度={speed}")
    except Exception as e:
        print(f"设置幻灯片切换效果失败: {e}")

# 26. 运行幻灯片放映
def run_slideshow(presentation):
    """
    运行演示文稿的幻灯片放映。
    :param presentation: 演示文稿对象
    """
    if not presentation:
        print("演示文稿对象无效。")
        return
    try:
        # 运行幻灯片放映
        presentation.SlideShowSettings.Run()
        print("幻灯片放映已启动。")
    except Exception as e:
        print(f"启动幻灯片放映失败: {e}")

# 27. 插入音频到幻灯片
def insert_audio_to_slide(slide, audio_file_path, left, top, width, height):
    """
    在幻灯片中插入音频。
    :param slide: 幻灯片对象
    :param audio_file_path: 音频文件的完整路径
    :param left: 音频图标左上角的 X 坐标
    :param top: 音频图标左上角的 Y 坐标
    :param width: 音频图标宽度
    :param height: 音频图标高度
    """
    if not slide:
        print("幻灯片对象无效。")
        return None
    try:
        shape = slide.Shapes.AddMediaObject2(audio_file_path, left, top, width, height)
        # 可以进一步设置音频的播放属性，例如自动播放
        # shape.MediaFormat.PlayOnEntry = True
        print(f"已在幻灯片中插入音频: {audio_file_path}")
        return shape
    except Exception as e:
        print(f"插入音频失败: {e}")
        return None

# 28. 插入视频到幻灯片
def insert_video_to_slide(slide, video_file_path, left, top, width, height):
    """
    在幻灯片中插入视频。
    :param slide: 幻灯片对象
    :param video_file_path: 视频文件的完整路径
    :param left: 视频左上角的 X 坐标
    :param top: 视频左上角的 Y 坐标
    :param width: 视频宽度
    :param height: 视频高度
    """
    if not slide:
        print("幻灯片对象无效。")
        return None
    try:
        shape = slide.Shapes.AddMediaObject2(video_file_path, left, top, width, height)
        # 可以进一步设置视频的播放属性，例如自动播放
        # shape.MediaFormat.PlayOnEntry = True
        print(f"已在幻灯片中插入视频: {video_file_path}")
        return shape
    except Exception as e:
        print(f"插入视频失败: {e}")
        return None

# 29. 复制幻灯片
def copy_slide(presentation, source_slide_index, destination_index):
    """
    复制指定索引的幻灯片到新位置。
    :param presentation: 演示文稿对象
    :param source_slide_index: 源幻灯片的索引 (从 1 开始)
    :param destination_index: 目标位置的索引 (从 1 开始)
    """
    if not presentation:
        print("演示文稿对象无效。")
        return
    try:
        # 获取源幻灯片
        source_slide = presentation.Slides(source_slide_index)
        # 复制幻灯片
        source_slide.Copy()
        # 粘贴幻灯片到指定位置
        presentation.Slides.Paste(destination_index)
        print(f"已复制幻灯片 {source_slide_index} 到 {destination_index}。")
    except Exception as e:
        print(f"复制幻灯片失败: {e}")

# 30. 移动幻灯片
def move_slide(presentation, source_index, destination_index):
    """
    移动幻灯片到新位置。
    :param presentation: 演示文稿对象
    :param source_index: 源幻灯片的索引 (从 1 开始)
    :param destination_index: 目标位置的索引 (从 1 开始)
    """
    if not presentation:
        print("演示文稿对象无效。")
        return
    try:
        # 移动幻灯片
        presentation.Slides.Move(source_index, destination_index)
        print(f"已将幻灯片 {source_index} 移动到 {destination_index}。")
    except Exception as e:
        print(f"移动幻灯片失败: {e}")

# 31. 获取幻灯片中所有形状的名称
def get_all_shape_names_on_slide(slide):
    """
    获取幻灯片中所有形状的名称。
    :param slide: 幻灯片对象
    :return: 形状名称列表
    """
    if not slide:
        print("幻灯片对象无效。")
        return []
    shape_names = []
    try:
        for shape in slide.Shapes:
            shape_names.append(shape.Name)
            print(f"形状名称: {shape.Name}")
        return shape_names
    except Exception as e:
        print(f"获取形状名称失败: {e}")
        return []

# 32. 改变形状的位置和大小
def set_shape_position_and_size(shape, left, top, width, height):
    """
    改变形状的位置和大小。
    :param shape: 形状对象
    :param left: 左上角的 X 坐标
    :param top: 左上角的 Y 坐标
    :param width: 宽度
    :param height: 高度
    """
    if not shape:
        print("形状对象无效。")
        return
    try:
        shape.Left = left
        shape.Top = top
        shape.Width = width
        shape.Height = height
        print(f"形状 '{shape.Name}' 位置和大小已调整: Left={left}, Top={top}, Width={width}, Height={height}")
    except Exception as e:
        print(f"设置形状位置和大小失败: {e}")

# 33. 设置形状的填充颜色
def set_shape_fill_color(shape, r, g, b):
    """
    设置形状的填充颜色。
    :param shape: 形状对象
    :param r: 红色分量 (0-255)
    :param g: 绿色分量 (0-255)
    :param b: 蓝色分量 (0-255)
    """
    if not shape:
        print("形状对象无效。")
        return
    try:
        fill = shape.Fill
        fill.ForeColor.RGB = r + (g << 8) + (b << 16)
        fill.Visible = True # 确保填充可见
        print(f"形状 '{shape.Name}' 填充颜色已设置为 RGB({r}, {g}, {b})。")
    except Exception as e:
        print(f"设置形状填充颜色失败: {e}")

# 34. 设置形状的边框颜色和粗细
def set_shape_line_format(shape, r, g, b, weight=1):
    """
    设置形状的边框颜色和粗细。
    :param shape: 形状对象
    :param r: 红色分量 (0-255)
    :param g: 绿色分量 (0-255)
    :param b: 蓝色分量 (0-255)
    :param weight: 边框粗细 (磅)
    """
    if not shape:
        print("形状对象无效。")
        return
    try:
        line = shape.Line
        line.ForeColor.RGB = r + (g << 8) + (b << 16)
        line.Weight = weight
        line.Visible = True # 确保边框可见
        print(f"形状 '{shape.Name}' 边框颜色已设置为 RGB({r}, {g}, {b}), 粗细为 {weight}。")
    except Exception as e:
        print(f"设置形状边框失败: {e}")

# 35. 将形状置于顶层或底层
def bring_shape_to_front_or_back(shape, to_front=True):
    """
    将形状置于顶层或底层。
    :param shape: 形状对象
    :param to_front: True 表示置于顶层, False 表示置于底层
    """
    if not shape:
        print("形状对象无效。")
        return
    try:
        if to_front:
            shape.ZOrder(0) # msoBringToFront
            print(f"形状 '{shape.Name}' 已置于顶层。")
        else:
            shape.ZOrder(4) # msoSendToBack
            print(f"形状 '{shape.Name}' 已置于底层。")
    except Exception as e:
        print(f"设置形状层级失败: {e}")

# 36. 添加一个矩形形状
def add_rectangle_to_slide(slide, left, top, width, height):
    """
    在幻灯片中添加一个矩形形状。
    :param slide: 幻灯片对象
    :param left: 矩形左上角的 X 坐标
    :param top: 矩形左上角的 Y 坐标
    :param width: 矩形宽度
    :param height: 矩形高度
    """
    if not slide:
        print("幻灯片对象无效。")
        return None
    try:
        # 添加一个矩形 (MsoShapeType.msoShapeRectangle = 1)
        shape = slide.Shapes.AddShape(1, left, top, width, height)
        print(f"已在幻灯片中添加一个矩形。")
        return shape
    except Exception as e:
        print(f"添加矩形失败: {e}")
        return None

# 37. 添加一个圆形/椭圆形形状
def add_oval_to_slide(slide, left, top, width, height):
    """
    在幻灯片中添加一个圆形/椭圆形形状。
    :param slide: 幻灯片对象
    :param left: 椭圆形左上角的 X 坐标
    :param top: 椭圆形左上角的 Y 坐标
    :param width: 椭圆形宽度
    :param height: 椭圆形高度
    """
    if not slide:
        print("幻灯片对象无效。")
        return None
    try:
        # 添加一个椭圆形 (MsoShapeType.msoShapeOval = 9)
        shape = slide.Shapes.AddShape(9, left, top, width, height)
        print(f"已在幻灯片中添加一个椭圆形。")
        return shape
    except Exception as e:
        print(f"添加椭圆形失败: {e}")
        return None

# 38. 添加一个箭头形状
def add_arrow_to_slide(slide, begin_x, begin_y, end_x, end_y):
    """
    在幻灯片中添加一个箭头形状。
    :param slide: 幻灯片对象
    :param begin_x: 箭头的起始 X 坐标
    :param begin_y: 箭头的起始 Y 坐标
    :param end_x: 箭头的结束 X 坐标
    :param end_y: 箭头的结束 Y 坐标
    """
    if not slide:
        print("幻灯片对象无效。")
        return None
    try:
        # 添加一条直线 (MsoShapeType.msoLine = 90)
        shape = slide.Shapes.AddConnector(1, begin_x, begin_y, end_x, end_y) # 1 为 msoConnectorStraight
        # 设置箭头类型
        line = shape.Line
        line.EndArrowheadStyle = 3 # msoArrowheadTriangle (三角形箭头)
        line.EndArrowheadLength = 3 # msoArrowheadLengthMedium
        line.EndArrowheadWidth = 3 # msoArrowheadWidthMedium
        print(f"已在幻灯片中添加一个箭头。")
        return shape
    except Exception as e:
        print(f"添加箭头失败: {e}")
        return None

# 39. 获取幻灯片中所有形状的类型
def get_all_shape_types_on_slide(slide):
    """
    获取幻灯片中所有形状的类型。
    :param slide: 幻灯片对象
    :return: 形状类型列表
    """
    if not slide:
        print("幻灯片对象无效。")
        return []
    shape_types = []
    try:
        for shape in slide.Shapes:
            shape_types.append(shape.Type)
            # 常见的类型值:
            # 1 (msoShapeRectangle)
            # 9 (msoShapeOval)
            # 17 (msoTextBox)
            # 13 (msoPicture)
            # 14 (msoPlaceholder)
            # 90 (msoLine)
            # 16 (msoTable)
            # 17 (msoChart)
            print(f"形状名称: {shape.Name}, 类型: {shape.Type}")
        return shape_types
    except Exception as e:
        print(f"获取形状类型失败: {e}")
        return []

# 40. 将幻灯片中的文本框文字替换
def replace_text_in_text_boxes(slide, old_text, new_text):
    """
    将幻灯片中所有文本框内的指定文字进行替换。
    :param slide: 幻灯片对象
    :param old_text: 要被替换的旧文本
    :param new_text: 替换成的新文本
    """
    if not slide:
        print("幻灯片对象无效。")
        return
    try:
        for shape in slide.Shapes:
            if shape.HasTextFrame and shape.TextFrame.HasText:
                text_range = shape.TextFrame.TextRange
                if old_text in text_range.Text:
                    text_range.Text = text_range.Text.replace(old_text, new_text)
                    print(f"在形状 '{shape.Name}' 中替换了文本: '{old_text}' -> '{new_text}'")
    except Exception as e:
        print(f"替换文本失败: {e}")

# 41. 获取演示文稿中所有幻灯片的数量
def get_presentation_slide_count(presentation):
    """
    获取演示文稿中所有幻灯片的数量。
    :param presentation: 演示文稿对象
    """
    if not presentation:
        print("演示文稿对象无效。")
        return 0
    try:
        count = presentation.Slides.Count
        print(f"演示文稿中共有 {count} 张幻灯片。")
        return count
    except Exception as e:
        print(f"获取幻灯片数量失败: {e}")
        return 0

# 42. 设置演示文稿的作者信息
def set_presentation_author(presentation, author_name):
    """
    设置演示文稿的作者信息。
    :param presentation: 演示文稿对象
    :param author_name: 作者姓名
    """
    if not presentation:
        print("演示文稿对象无效。")
        return
    try:
        presentation.BuiltInDocumentProperties("Author").Value = author_name
        print(f"演示文稿作者已设置为: '{author_name}'")
    except Exception as e:
        print(f"设置演示文稿作者失败: {e}")

# 43. 获取演示文稿的作者信息
def get_presentation_author(presentation):
    """
    获取演示文稿的作者信息。
    :param presentation: 演示文稿对象
    :return: 作者姓名
    """
    if not presentation:
        print("演示文稿对象无效。")
        return ""
    try:
        author = presentation.BuiltInDocumentProperties("Author").Value
        print(f"演示文稿作者是: '{author}'")
        return author
    except Exception as e:
        print(f"获取演示文稿作者失败: {e}")
        return ""

# 44. 复制幻灯片中的形状
def copy_shape_on_slide(slide, shape_name):
    """
    复制幻灯片中的指定形状。
    :param slide: 幻灯片对象
    :param shape_name: 要复制的形状名称
    """
    if not slide:
        print("幻灯片对象无效。")
        return
    try:
        shape_to_copy = find_shape_by_name(slide, shape_name)
        if shape_to_copy:
            shape_to_copy.Copy()
            print(f"已复制形状 '{shape_name}'。")
        else:
            print(f"未找到要复制的形状 '{shape_name}'。")
    except Exception as e:
        print(f"复制形状失败: {e}")

# 45. 粘贴形状到幻灯片
def paste_shape_to_slide(slide, left, top):
    """
    粘贴形状到幻灯片。
    :param slide: 幻灯片对象
    :param left: 粘贴位置的 X 坐标
    :param top: 粘贴位置的 Y 坐标
    """
    if not slide:
        print("幻灯片对象无效。")
        return None
    try:
        # 粘贴形状，并获取新粘贴的形状
        pasted_shape = slide.Shapes.Paste()
        # 设置粘贴位置
        pasted_shape.Left = left
        pasted_shape.Top = top
        print(f"形状已粘贴到幻灯片，位置: Left={left}, Top={top}。")
        return pasted_shape
    except Exception as e:
        print(f"粘贴形状失败: {e}")
        return None

# 46. 设置幻灯片视图模式
def set_slide_view_mode(ppt_app, view_type=1):
    """
    设置 PowerPoint 应用程序的幻灯片视图模式。
    :param ppt_app: PowerPoint 应用程序对象
    :param view_type: 视图类型 (例如: 1 为普通视图，更多类型详见 MSO.PpViewType 枚举)
                      ppViewNormal = 1
                      ppViewSlideSorter = 2
                      ppViewNotesPage = 3
                      ppViewOutline = 4
                      ppViewSlide = 5
                      ppViewHandoutMaster = 6
                      ppViewNotesMaster = 7
                      ppViewOutlineBrowse = 8
                      ppViewMasterThumbnails = 9
                      ppViewPrintPreview = 10
                      ppViewNormalView = 11 (Office 2013 及更高版本)
    """
    if not ppt_app:
        print("PowerPoint 应用程序未启动。")
        return
    try:
        ppt_app.ActiveWindow.View.Type = view_type
        print(f"PowerPoint 视图模式已设置为: {view_type}")
    except Exception as e:
        print(f"设置视图模式失败: {e}")

# 47. 隐藏或显示幻灯片
def hide_or_show_slide(slide, hide=True):
    """
    隐藏或显示幻灯片。
    :param slide: 幻灯片对象
    :param hide: True 表示隐藏, False 表示显示
    """
    if not slide:
        print("幻灯片对象无效。")
        return
    try:
        slide.SlideShowTransition.Hidden = hide
        status = "隐藏" if hide else "显示"
        print(f"幻灯片 '{slide.SlideIndex}' 已设置为 {status}。")
    except Exception as e:
        print(f"隐藏/显示幻灯片失败: {e}")

# 48. 添加超链接到文本框
def add_hyperlink_to_text_box(text_box_shape, address, text_to_display=None):
    """
    给文本框添加超链接。
    :param text_box_shape: 文本框形状对象
    :param address: 超链接地址 (例如: "http://www.google.com" 或 "C:\\temp\\document.pdf")
    :param text_to_display: 要显示的文本 (可选, 如果不提供则使用文本框现有文本)
    """
    if not text_box_shape or not hasattr(text_box_shape, 'TextFrame'):
        print("形状对象无效或不是文本框。")
        return
    try:
        text_range = text_box_shape.TextFrame.TextRange
        if text_to_display:
            text_range.Text = text_to_display
        
        # 添加超链接
        text_range.ActionSettings[1].Hyperlink.Address = address # 1 for msoActionClick
        text_range.ActionSettings[1].Action = 1 # msoActionHyperlink
        
        print(f"已为文本框 '{text_box_shape.Name}' 添加超链接: {address}")
    except Exception as e:
        print(f"添加超链接失败: {e}")

# 49. 在幻灯片中添加备注
def add_notes_to_slide(slide, notes_text):
    """
    在幻灯片中添加备注。
    :param slide: 幻灯片对象
    :param notes_text: 备注内容
    """
    if not slide:
        print("幻灯片对象无效。")
        return
    try:
        # 获取备注页面
        notes_page = slide.NotesPage
        # 获取备注文本框 (通常是 NotesPage 中的一个文本框)
        for shape in notes_page.Shapes:
            if shape.HasTextFrame and shape.TextFrame.HasText:
                # 假设第一个文本框是备注文本框
                shape.TextFrame.TextRange.Text = notes_text
                print(f"已为幻灯片 {slide.SlideIndex} 添加备注。")
                return
        print(f"未找到幻灯片 {slide.SlideIndex} 的备注文本框。")
    except Exception as e:
        print(f"添加备注失败: {e}")

# 50. 检查演示文稿是否已保存
def check_presentation_saved_status(presentation):
    """
    检查演示文稿是否已保存。
    :param presentation: 演示文稿对象
    :return: True 如果已保存, False 如果未保存
    """
    if not presentation:
        print("演示文稿对象无效。")
        return False
    try:
        saved = presentation.Saved
        if saved:
            print("演示文稿已保存。")
        else:
            print("演示文稿未保存 (有未保存的更改)。")
        return saved
    except Exception as e:
        print(f"检查保存状态失败: {e}")
        return False


# --- 使用示例 ---
if __name__ == "__main__":
    ppt_app = None
    presentation = None
    try:
        # 1. 启动 PowerPoint 应用程序
        ppt_app = create_ppt_application()
        if not ppt_app:
            exit()

        # 2. 创建一个新的演示文稿
        presentation = create_new_presentation(ppt_app)
        if not presentation:
            quit_ppt_application(ppt_app)
            exit()

        # 8. 添加一张新的幻灯片 (标题幻灯片)
        slide1 = add_new_slide(presentation, 1)
        if slide1:
            # 18. 修改幻灯片标题
            set_slide_title(slide1, "Pywin32 操作 PPT 示例")
            # 11. 在幻灯片中添加文本框
            subtitle_shape = add_text_box_to_slide(slide1, "用 Python 自动化您的演示文稿！", 100, 200, 600, 50)
            if subtitle_shape:
                # 12. 设置文本框字体样式
                set_text_box_font(subtitle_shape, font_name="Arial", font_size=36, bold=True, italic=True)

        # 8. 添加一张新的幻灯片 (标题和内容幻灯片)
        slide2 = add_new_slide(presentation, 2)
        if slide2:
            set_slide_title(slide2, "插入图片和形状")
            # 11. 添加内容文本框
            content_shape = add_text_box_to_slide(slide2, "这张幻灯片演示了如何插入图片和自定义形状。", 50, 100, 600, 50)
            # 13. 在幻灯片中插入图片
            # 假设你有一个图片文件在当前目录下
            import os
            current_dir = os.path.dirname(os.path.abspath(__file__))
            picture_path = os.path.join(current_dir, "example_image.png") # 请确保存在此图片文件
            # 为了演示，创建一个虚拟图片文件
            from PIL import Image
            img = Image.new('RGB', (60, 30), color = 'red')
            img.save(picture_path)

            if os.path.exists(picture_path):
                insert_picture_to_slide(slide2, picture_path, 50, 150, 300, 200)
            else:
                print(f"图片文件 {picture_path} 不存在，跳过插入图片。")

            # 36. 添加一个矩形形状
            rect_shape = add_rectangle_to_slide(slide2, 400, 150, 150, 100)
            if rect_shape:
                # 33. 设置形状的填充颜色 (蓝色)
                set_shape_fill_color(rect_shape, 0, 0, 255)
                # 34. 设置形状的边框颜色和粗细 (红色边框，粗细 3 磅)
                set_shape_line_format(rect_shape, 255, 0, 0, 3)

            # 37. 添加一个圆形/椭圆形形状
            oval_shape = add_oval_to_slide(slide2, 450, 300, 100, 100)
            if oval_shape:
                # 33. 设置形状的填充颜色 (绿色)
                set_shape_fill_color(oval_shape, 0, 255, 0)
                # 35. 将形状置于顶层
                bring_shape_to_front_or_back(oval_shape, True)

            # 38. 添加一个箭头形状
            arrow_shape = add_arrow_to_slide(slide2, 300, 250, 450, 300)
            if arrow_shape:
                # 34. 设置箭头颜色 (黑色)
                set_shape_line_format(arrow_shape, 0, 0, 0, 2)


        # 8. 添加一张新的幻灯片 (空白幻灯片)
        slide3 = add_new_slide(presentation, 12) # ppLayoutBlank
        if slide3:
            set_slide_title(slide3, "表格和图表示例") # 空白幻灯片可能没有标题占位符
            # 11. 手动添加一个标题文本框
            title_shape_3 = add_text_box_to_slide(slide3, "表格和图表示例", 50, 50, 600, 50)
            if title_shape_3:
                set_text_box_font(title_shape_3, font_size=40, bold=True)


            # 19. 插入表格到幻灯片
            table_shape = insert_table_to_slide(slide3, 3, 3, 50, 150, 300, 150)
            if table_shape:
                # 20. 设置表格单元格文本
                set_table_cell_text(table_shape, 1, 1, "姓名")
                set_table_cell_text(table_shape, 1, 2, "年龄")
                set_table_cell_text(table_shape, 1, 3, "城市")
                set_table_cell_text(table_shape, 2, 1, "张三")
                set_table_cell_text(table_shape, 2, 2, "30")
                set_table_cell_text(table_shape, 2, 3, "北京")
                set_table_cell_text(table_shape, 3, 1, "李四")
                set_table_cell_text(table_shape, 3, 2, "25")
                set_table_cell_text(table_shape, 3, 3, "上海")

            # 21. 插入图表到幻灯片 (簇状柱形图)
            # 使用 wc.constants 获取 Office 常量
            chart_shape = insert_chart_to_slide(slide3, wc.constants.xlColumnClustered, 400, 150, 400, 300)
            if chart_shape:
                # 22. 更新图表数据
                update_chart_data(chart_shape, 1, [100, 200, 150, 250])
                # 23. 修改图表标题
                set_chart_title(chart_shape, "销售额图表")

        # 8. 添加一张新的幻灯片 (标题和内容)
        slide4 = add_new_slide(presentation, 2)
        if slide4:
            set_slide_title(slide4, "高级操作")
            # 25. 设置幻灯片切换效果 (淡入淡出，中速)
            set_slide_transition(slide4, 3, 2) # ppEffectFade, ppTransitionMedium

            # 40. 将幻灯片中的文本框文字替换
            # 先添加一个文本框
            replace_text_box = add_text_box_to_slide(slide4, "原始文本包含'旧'文字。", 50, 150, 400, 50)
            if replace_text_box:
                replace_text_in_text_boxes(slide4, "旧", "新")

            # 48. 添加超链接到文本框
            hyperlink_text_box = add_text_box_to_slide(slide4, "点击访问 Google", 50, 250, 200, 30)
            if hyperlink_text_box:
                add_hyperlink_to_text_box(hyperlink_text_box, "http://www.google.com")

            # 49. 在幻灯片中添加备注
            add_notes_to_slide(slide4, "这是第四张幻灯片的备注，用于演示添加备注功能。")

        # 29. 复制幻灯片 (复制第一张到末尾)
        copy_slide(presentation, 1, presentation.Slides.Count + 1)
        print(f"当前幻灯片总数: {get_presentation_slide_count(presentation)}")

        # 30. 移动幻灯片 (将复制的幻灯片移动到第二张)
        move_slide(presentation, presentation.Slides.Count, 2)
        print(f"移动幻灯片后，当前幻灯片总数: {get_presentation_slide_count(presentation)}")

        # 7. 获取当前活动演示文稿
        get_active_presentation(ppt_app)

        # 42. 设置演示文稿的作者信息
        set_presentation_author(presentation, "Pywin32 自动化演示")
        # 43. 获取演示文稿的作者信息
        get_presentation_author(presentation)

        # 46. 设置幻灯片视图模式
        set_slide_view_mode(ppt_app, 2) # ppViewSlideSorter

        # 47. 隐藏幻灯片 (隐藏第三张)
        hide_or_show_slide(presentation.Slides(3), True)

        # 41. 获取演示文稿中所有幻灯片的数量
        get_presentation_slide_count(presentation)

        # 4. 保存演示文稿
        save_path = os.path.join(current_dir, "MyAutomatedPresentation.pptx")
        save_presentation(presentation, save_path)

        # 50. 检查演示文稿是否已保存
        check_presentation_saved_status(presentation)

        # 26. 运行幻灯片放映 (可选，会启动放映模式)
        # run_slideshow(presentation)

        # 稍作等待，以便观察效果
        # import time
        # time.sleep(5)

    except Exception as e:
        print(f"发生错误: {e}")
    finally:
        # 5. 关闭演示文稿
        if presentation:
            close_presentation(presentation)
        # 6. 退出 PowerPoint 应用程序
        if ppt_app:
            quit_ppt_application(ppt_app)


#pip install streamlit pandas PyPDF2 python-docx python-pptx openpyxl xlrd textract pypandoc pillow   -i   https://pypi.tuna.tsinghua.edu.cn/simple



# 忽略特定警告
import warnings
warnings.filterwarnings("ignore", category=UserWarning)

# --- 尝试导入各函数库，并追踪其可用性 ---
libraries_available = {
    "pandas": False,
    "PyPDF2": False,
    "docx": False,
    "pptx": False,
    "openpyxl": False,
    "xlrd": False,
    "textract": False,
    "pypandoc": False
}

import streamlit as st
import tempfile
import os
import shutil
import time
import io
import subprocess

# 尝试导入 pandas
try:
    import pandas as pd
    libraries_available["pandas"] = True
except ImportError:
    pass

# 尝试导入 PyPDF2
try:
    import PyPDF2
    libraries_available["PyPDF2"] = True
except ImportError:
    pass

# 尝试导入 python-docx
try:
    from docx import Document
    libraries_available["docx"] = True
except ImportError:
    pass

# 尝试导入 python-pptx
try:
    from pptx import Presentation
    libraries_available["pptx"] = True
except ImportError:
    pass

# 尝试导入 openpyxl
try:
    import openpyxl
    libraries_available["openpyxl"] = True
except ImportError:
    pass

# 尝试导入 xlrd (用于旧版 Excel)
try:
    import xlrd
    libraries_available["xlrd"] = True
except ImportError:
    pass

# 尝试导入 textract (用于提取文本)
try:
    import textract
    libraries_available["textract"] = True
except ImportError:
    pass

# 尝试导入 pypandoc (用于文档转换)
try:
    import pypandoc
    # 确保 pandoc 已安装
    try:
        pypandoc.get_pandoc_path()
        libraries_available["pypandoc"] = True
    except OSError:
        pass
except ImportError:
    pass

# Pillow 函数库检查 - 必需库
try:
    from PIL import Image
except ImportError:
    st.error("❌ **严重错误：** 缺少 `Pillow` 函数库。\n"
             "图片显示与处理功能将无法使用。\n"
             "请运行 `pip install Pillow`。")
    st.stop()

# --- 显示所有缺失库的警告 ---
if not libraries_available["pandas"]:
    st.warning("⚠️ **警告：** 缺少 `pandas` 函数库。\n"
               "Excel 文件 (`.xls`, `.xlsx`) 处理功能将无法使用。\n"
               "请运行 `pip install pandas`。")

if not libraries_available["PyPDF2"]:
    st.warning("⚠️ **警告：** 缺少 `PyPDF2` 函数库。\n"
               "PDF 文件处理功能将无法使用。\n"
               "请运行 `pip install PyPDF2`。")

if not libraries_available["docx"]:
    st.warning("⚠️ **警告：** 缺少 `python-docx` 函数库。\n"
               "Word (.docx) 文件处理功能将无法使用。\n"
               "请运行 `pip install python-docx`。")

if not libraries_available["pptx"]:
    st.warning("⚠️ **警告：** 缺少 `python-pptx` 函数库。\n"
               "PowerPoint (.pptx) 文件处理功能将无法使用。\n"
               "请运行 `pip install python-pptx`。")

if not libraries_available["openpyxl"]:
    if libraries_available["pandas"]:
        st.warning("⚠️ **警告：** 缺少 `openpyxl` 函数库。\n"
                   "Excel 的高级功能（如图片提取）可能受限。\n"
                   "请运行 `pip install openpyxl`。")

if not libraries_available["xlrd"]:
    st.warning("⚠️ **警告：** 缺少 `xlrd` 函数库。\n"
               "旧版 Excel (.xls) 文件处理功能将无法使用。\n"
               "请运行 `pip install xlrd`。")

if not libraries_available["textract"]:
    st.warning("⚠️ **警告：** 缺少 `textract` 函数库。\n"
               "部分文件格式的文本提取功能将受限。\n"
               "请运行 `pip install textract`。")

if not libraries_available["pypandoc"]:
    st.warning("⚠️ **警告：** 缺少 `pypandoc` 函数库或 `pandoc` 命令行工具。\n"
               "文档转换功能将受限。\n"
               "请安装:\n"
               "1. `pip install pypandoc`\n"
               "2. 从 https://pandoc.org/installing.html 安装 pandoc")

# --- Streamlit 应用程序界面 ---
st.set_page_config(layout="wide")
st.title("通用文件上传与处理工具")
st.write("请上传图片、音讯、视讯、Excel、PDF、Word、PowerPoint 或文字文件，\n"
         "我会尝试处理并显示其内容。")

uploaded_file = st.file_uploader("选择一个文件上传", type=[
    "jpg", "jpeg", "png", "gif",            # 图片
    "mp3", "wav", "ogg",                    # 音讯
    "mp4", "mov", "avi",                    # 视讯
    "xls", "xlsx",                          # Excel
    "pdf",                                  # PDF
    "doc", "docx",                          # Word
    "ppt", "pptx",                          # PowerPoint
    "txt"                                   # 文字文件
])

# 初始化变量
temp_file_path = None
converted_file_path = None

def convert_with_pandoc(input_path, output_extension):
    """使用 pandoc 转换文件格式"""
    try:
        output_path = tempfile.mktemp(suffix=output_extension)
        pypandoc.convert_file(
            input_path,
            output_extension[1:],  # 格式如 'docx'
            outputfile=output_path
        )
        return output_path
    except Exception as e:
        st.error(f"文件转换失败: {str(e)}")
        return None

def extract_text_with_textract(file_path):
    """使用 textract 提取文件文本"""
    try:
        text = textract.process(file_path).decode('utf-8')
        return text
    except Exception as e:
        st.error(f"文本提取失败: {str(e)}")
        return None

def is_pandoc_available():
    """检查 pandoc 是否可用"""
    try:
        result = subprocess.run(["pandoc", "--version"], capture_output=True, text=True)
        return result.returncode == 0
    except:
        return False

if uploaded_file:
    # 获取文件的 MIME 类型
    file_type = uploaded_file.type
    st.info(f"检测到的文件类型: **{file_type}**")
    st.info(f"文件名称: **{uploaded_file.name}**")

    # 保存上传文件到临时位置
    file_extension = os.path.splitext(uploaded_file.name)[1].lower()

    with tempfile.NamedTemporaryFile(delete=False, suffix=file_extension) as temp_file:
        temp_file.write(uploaded_file.read())
        temp_file_path = temp_file.name

    st.success(f"文件已暂存至: `{temp_file_path}`")

    # 根据文件类型处理文件
    # 图片文件
    if file_type.startswith("image/"):
        st.subheader("🖼️ 图片文件处理")
        try:
            image = Image.open(temp_file_path)
            st.image(image,
                     caption=f"上传图片: {uploaded_file.name}",
                     use_container_width=True)
            st.write(f"图片尺寸: **{image.size[0]} x {image.size[1]} 像素**")
            st.write(f"图片格式: **{image.format}**")
            image.close()
        except Exception as e:
            st.error(f"处理图片文件时发生错误: {e}")

    # 音讯文件
    elif file_type.startswith("audio/"):
        st.subheader("🎵 音讯文件处理")
        st.audio(temp_file_path, format=file_type)
        st.info("Streamlit 内置播放器会尝试播放音讯。\n"
                "若需更复杂的音讯处理，需使用专用 Python 函数库。")

    # 视讯文件
    elif file_type.startswith("video/"):
        st.subheader("🎬 视讯文件处理")
        st.video(temp_file_path, format=file_type)
        st.info("Streamlit 内置播放器会尝试播放视讯。\n"
                "若需视讯分析或编辑，需使用如 OpenCV 等函数库。")

    # Excel 文件 (xls, xlsx)
    elif file_type == "application/vnd.ms-excel" or \
         file_type == "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet":
        st.subheader("📊 Excel 文件处理")
        
        # 处理旧版 .xls 文件
        if file_extension == ".xls":
            if libraries_available["xlrd"]:
                try:
                    # 使用 xlrd 读取旧版 Excel
                    workbook = xlrd.open_workbook(temp_file_path)
                    sheet_names = workbook.sheet_names()
                    
                    st.success("Excel 文件读取成功！")
                    st.write(f"此 Excel 文件包含 **{len(sheet_names)} 个工作表**。")
                    
                    selected_sheet = st.selectbox("选择要查看的工作表:",
                                                  sheet_names,
                                                  index=0,
                                                  key="excel_sheet_select")
                    
                    sheet = workbook.sheet_by_name(selected_sheet)
                    
                    # 显示表格数据
                    data = []
                    for row_idx in range(0, min(sheet.nrows, 100)):
                        row_data = []
                        for col_idx in range(sheet.ncols):
                            cell_value = sheet.cell_value(row_idx, col_idx)
                            row_data.append(cell_value)
                        data.append(row_data)
                    
                    st.write(f"**显示工作表: `{selected_sheet}` 的内容 (预览前 100 行)**")
                    st.dataframe(data, use_container_width=True)
                    
                    # 提取文本内容
                    text_content = []
                    for row in data:
                        for cell in row:
                            if isinstance(cell, (str, int, float)) and str(cell).strip():
                                text_content.append(str(cell).strip())
                    
                    if text_content:
                        with st.expander("点击查看所有提取的单元格文字"):
                            st.text_area("提取的文字:",
                                         "\n---\n".join(text_content),
                                         height=300)
                    else:
                        st.info("此工作表的单元格中未找到任何显著的文字内容。")
                    
                except Exception as e:
                    st.error(f"处理旧版 Excel 文件时发生错误: {e}")
            else:
                st.warning("缺少 `xlrd` 库，无法处理旧版 Excel (.xls) 文件。")
        
        # 处理新版 .xlsx 文件
        elif file_extension == ".xlsx" and libraries_available["pandas"]:
            excel_file_handle = None
            temp_image_dir = None
            try:
                excel_file_handle = pd.ExcelFile(temp_file_path)
                sheet_names = excel_file_handle.sheet_names

                st.success("Excel 文件读取成功！")
                st.write(f"此 Excel 文件包含 **{len(sheet_names)} 个工作表**。")

                selected_sheet = st.selectbox("选择要查看的工作表:",
                                              sheet_names,
                                              index=0,
                                              key="excel_sheet_select")

                df_selected = pd.read_excel(excel_file_handle,
                                            sheet_name=selected_sheet)

                st.write(f"**显示工作表: `{selected_sheet}` 的内容 (预览前 100 行)**")
                st.dataframe(df_selected.head(100), use_container_width=True)

                if len(df_selected) > 100:
                    if st.checkbox(f"点击查看 `{selected_sheet}` 的完整表格\n"
                                   "(可能需要更多内存和时间)",
                                   key=f"show_full_df_{selected_sheet}"):
                        st.dataframe(df_selected, use_container_width=True)

                st.subheader(f"📝 工作表: `{selected_sheet}` 中的文字内容")
                text_content = []
                for col in df_selected.columns:
                    for cell_value in df_selected[col].astype(str):
                        if pd.notna(cell_value) and len(cell_value.strip()) > 0:
                            text_content.append(cell_value.strip())

                if text_content:
                    with st.expander("点击查看所有提取的单元格文字"):
                        st.text_area("提取的文字:",
                                     "\n---\n".join(text_content),
                                     height=300)
                else:
                    st.info("此工作表的单元格中未找到任何显著的文字内容。")

                # 图片提取部分
                if libraries_available["openpyxl"]:
                    st.subheader(f"🖼️ 工作表: `{selected_sheet}` 中的图片")
                    image_found = False
                    try:
                        temp_image_dir = tempfile.mkdtemp()
                        workbook = openpyxl.load_workbook(temp_file_path)
                        
                        # 使用更健壮的方式获取工作表
                        try:
                            sheet = workbook[selected_sheet]
                        except KeyError:
                            sheet = None
                            for ws in workbook.worksheets:
                                if ws.title == selected_sheet:
                                    sheet = ws
                                    break
                            
                            if sheet is None:
                                st.error(f"找不到名称为 '{selected_sheet}' 的工作表")
                                images = []
                            else:
                                images = getattr(sheet, '_images', [])
                        else:
                            images = getattr(sheet, '_images', [])
                        
                        if images:
                            total_images = len(images)
                            max_images_to_preview = 5

                            st.write(f"在 `{selected_sheet}` 中找到 **{total_images}** 张图片。")

                            if total_images > max_images_to_preview:
                                st.info(f"为了性能，**仅预览前 {max_images_to_preview} 张图片**。")
                                if st.checkbox("显示所有图片 (可能导致卡顿，请谨慎)",
                                               key=f"show_all_images_{selected_sheet}"):
                                    max_images_to_preview = total_images

                            for i, img in enumerate(images):
                                if i >= max_images_to_preview:
                                    break

                                # 尝试确定图片扩展名
                                image_ext = ".png"
                                if hasattr(img, 'format') and img.format:
                                    if img.format.lower() in ['png', 'jpeg', 'gif']:
                                        image_ext = f".{img.format.lower()}"
                                
                                image_filename = f"image_{i+1}{image_ext}"
                                image_path = os.path.join(temp_image_dir, image_filename)

                                try:
                                    # 正确获取图片数据
                                    if hasattr(img, '_data'):
                                        pil_image = Image.open(io.BytesIO(img._data()))
                                        
                                        # 转换图片模式以确保兼容性
                                        if pil_image.mode in ('RGBA', 'P'):
                                            pil_image = pil_image.convert('RGB')
                                        
                                        # 保存图片
                                        pil_image.save(image_path)
                                        st.image(image_path,
                                                 caption=f"图片 {i+1} 来自 {selected_sheet}",
                                                 use_container_width=True)
                                        image_found = True
                                    else:
                                        st.warning(f"图片 {i+1} 缺少数据，无法显示")
                                except Exception as img_err:
                                    st.warning(f"无法保存图片 {i+1}。错误: {img_err}")

                            if image_found:
                                st.success(f"成功提取并显示 {min(total_images, max_images_to_preview)} 张图片")
                            else:
                                st.info("此工作表中找到图片但无法显示")
                        else:
                            st.info("此工作表中未找到任何嵌入图片。")

                    except Exception as e:
                        st.error(f"从Excel文件中提取图片时发生错误: {e}")
                    finally:
                        if temp_image_dir and os.path.exists(temp_image_dir):
                            try:
                                shutil.rmtree(temp_image_dir)
                            except Exception as e:
                                st.warning(f"无法删除临时图片目录 {temp_image_dir}。原因: {e}")
                else:
                    st.warning("`openpyxl`库未加载，无法进行Excel图片提取。")
            except Exception as e:
                st.error(f"处理 Excel 文件时发生错误: {e}")
            finally:
                if excel_file_handle:
                    excel_file_handle.close()
        else:
            st.warning("无法处理此 Excel 文件格式。")

    # PDF 文件
    elif file_type == "application/pdf":
        st.subheader("📄 PDF 文件处理")
        if libraries_available["PyPDF2"]:
            pdf_file = None
            try:
                pdf_file = open(temp_file_path, "rb")
                reader = PyPDF2.PdfReader(pdf_file)
                num_pages = len(reader.pages)
                st.write(f"PDF 文件总页数: **{num_pages}**")

                extracted_text = []
                for i in range(min(num_pages, 3)):
                    page = reader.pages[i]
                    text = page.extract_text()
                    if text:
                        extracted_text.append(f"--- 第 {i+1} 页 ---\n"
                                              f"{text[:500]}..." if len(text) > 500 else
                                              f"--- 第 {i+1} 页 ---\n{text}")
                    else:
                        extracted_text.append(f"--- 第 {i+1} 页 ---\n"
                                              "(无法提取文字，可能为扫描件或图像式 PDF)")

                if extracted_text:
                    st.text_area("部分文字内容预览:",
                                 "\n\n".join(extracted_text),
                                 height=300)
                else:
                    st.info("无法从 PDF 中提取任何文字内容。")

            except Exception as e:
                st.error(f"处理 PDF 文件时发生错误: {e}")
                st.info("注意: `PyPDF2` 主要用于文字型 PDF，\n"
                        "可能无法从扫描或图像型 PDF 中提取文字。")
            finally:
                if pdf_file:
                    pdf_file.close()
        else:
            st.warning("`PyPDF2` 函数库未加载，无法处理 PDF 文件。")

    # Word 文件 (.doc 和 .docx)
    elif file_type == "application/vnd.openxmlformats-officedocument.wordprocessingml.document" or \
         file_type == "application/msword":
        st.subheader("📝 Word 文件处理")
        
        # 处理 .docx 文件
        if file_extension == ".docx" and libraries_available["docx"]:
            try:
                document = Document(temp_file_path)
                full_text = []
                for para in document.paragraphs:
                    full_text.append(para.text)

                doc_text = "\n".join(full_text)
                if doc_text:
                    st.write(f"文件总段落数: **{len(document.paragraphs)}**")
                    st.text_area("文件内容预览:",
                                 doc_text[:1000] + "..." if len(doc_text) > 1000 else doc_text,
                                 height=300)
                else:
                    st.info("无法从 Word 文件中提取任何文字内容。")
            except Exception as e:
                st.error(f"处理 Word 文件时发生错误: {e}")
        
        # 处理旧版 .doc 文件
        elif file_extension == ".doc":
            st.warning("检测到旧版 Word (.doc) 文件，尝试提取文本内容...")
            
            # 尝试使用 pandoc 转换为 docx
            if libraries_available["pypandoc"] and is_pandoc_available():
                st.info("尝试使用 pandoc 转换文件...")
                try:
                    converted_file_path = convert_with_pandoc(temp_file_path, ".docx")
                    if converted_file_path and os.path.exists(converted_file_path):
                        st.success("文件转换成功！")
                        
                        # 尝试处理转换后的 docx 文件
                        try:
                            document = Document(converted_file_path)
                            full_text = []
                            for para in document.paragraphs:
                                full_text.append(para.text)

                            doc_text = "\n".join(full_text)
                            if doc_text:
                                st.text_area("文件内容预览:",
                                             doc_text[:1000] + "..." if len(doc_text) > 1000 else doc_text,
                                             height=300)
                            else:
                                st.info("无法从转换后的文件中提取任何文字内容。")
                        except Exception as e:
                            st.error(f"处理转换后的文件时发生错误: {e}")
                except Exception as e:
                    st.error(f"文件转换失败: {str(e)}")
            
            # 如果 pandoc 不可用，尝试使用 textract
            elif libraries_available["textract"]:
                st.info("尝试使用 textract 提取文本...")
                try:
                    text = extract_text_with_textract(temp_file_path)
                    if text:
                        st.text_area("提取的文本内容:",
                                     text[:2000] + "..." if len(text) > 2000 else text,
                                     height=300)
                    else:
                        st.info("无法从 .doc 文件中提取文本内容。")
                except Exception as e:
                    st.error(f"提取文本时出错: {e}")
            
            else:
                st.warning("无法处理旧版 .doc 文件。请安装:\n"
                           "1. `pip install textract` (需要系统安装 antiword)\n"
                           "2. `pip install pypandoc` 并安装 pandoc")
        
        else:
            st.warning("无法处理此 Word 文件格式。")

    # PowerPoint 文件 (.ppt 和 .pptx)
    elif file_type == "application/vnd.openxmlformats-officedocument.presentationml.presentation" or \
         file_type == "application/vnd.ms-powerpoint":
        st.subheader("💡 PowerPoint 文件处理")
        
        # 处理 .pptx 文件
        if file_extension == ".pptx" and libraries_available["pptx"]:
            try:
                prs = Presentation(temp_file_path)
                total_slides = len(prs.slides)
                st.write(f"简报总页数: **{total_slides}**")

                presentation_text = []
                for i, slide in enumerate(prs.slides):
                    slide_text = []
                    for shape in slide.shapes:
                        if hasattr(shape, "text"):
                            slide_text.append(shape.text)
                    if slide_text:
                        presentation_text.append(f"--- 投影片 {i+1} ---\n" + "\n".join(slide_text))

                if presentation_text:
                    st.text_area("部分简报内容预览:",
                                 "\n\n".join(presentation_text[:min(total_slides, 3)]),
                                 height=300)
                else:
                    st.info("无法从 PowerPoint 文件中提取任何文字内容。")
            except Exception as e:
                st.error(f"处理 PowerPoint 文件时发生错误: {e}")
        
        # 处理旧版 .ppt 文件
        elif file_extension == ".ppt":
            st.warning("检测到旧版 PowerPoint (.ppt) 文件，尝试提取文本内容...")
            
            # 尝试使用 textract 提取文本
            if libraries_available["textract"]:
                st.info("尝试使用 textract 提取文本...")
                try:
                    text = extract_text_with_textract(temp_file_path)
                    if text:
                        st.text_area("提取的文本内容:",
                                     text[:2000] + "..." if len(text) > 2000 else text,
                                     height=300)
                    else:
                        st.info("无法从 .ppt 文件中提取文本内容。")
                except Exception as e:
                    st.error(f"提取文本时出错: {e}")
            else:
                st.warning("无法处理旧版 .ppt 文件。请安装 textract 库：`pip install textract`")
        
        else:
            st.warning("无法处理此 PowerPoint 文件格式。")

    # 文字文件
    elif file_type.startswith("text/"):
        st.subheader("📜 文字文件处理")
        try:
            # 尝试多种编码
            encodings = ['utf-8', 'latin-1', 'cp1252', 'gbk', 'gb2312', 'big5']
            content = None
            used_encoding = None
            
            for encoding in encodings:
                try:
                    with open(temp_file_path, "r", encoding=encoding) as text_file:
                        content = text_file.read()
                    used_encoding = encoding
                    break
                except UnicodeDecodeError:
                    continue
            
            if content is None:
                st.error("无法使用任何常见编码读取文件")
            else:
                st.success(f"使用 {used_encoding} 编码成功读取文件")
                st.write(f"文件大小: **{len(content.encode('utf-8')) / 1024:.2f} KB**")
                st.text_area("文件内容预览:",
                             content[:2000] + "..." if len(content) > 2000 else content,
                             height=300)
        except Exception as e:
            st.error(f"处理文字文件时发生错误: {e}")

    # 其他不支持的档案类型
    else:
        st.warning(f"🤔 抱歉，目前不支持处理类型为 `{file_type}` 的档案。")

    # --- 清理暂存文件 ---
    if temp_file_path and os.path.exists(temp_file_path):
        try:
            os.unlink(temp_file_path)
        except Exception as e:
            st.warning(f"删除原始暂存文件时出错: {e}")
    
    if converted_file_path and os.path.exists(converted_file_path):
        try:
            os.unlink(converted_file_path)
        except Exception as e:
            st.warning(f"删除转换后文件时出错: {e}")

else:
    st.info("👆 请上传文件以开始处理")


import platform
import os
import sys
import wmi
import subprocess

def list_network_adapters():
    system_type = platform.system()
    if system_type == 'Windows':
        return list_network_adapters_windows()
    elif system_type in ['Linux', 'Darwin']:
        return list_network_adapters_unix()
    else:
        print("不支持的操作系统")
        return []

def list_network_adapters_windows():
    c = wmi.WMI()
    adapters = []
    for adapter in c.Win32_NetworkAdapterConfiguration(IPEnabled=True):
        ip_address = ', '.join(adapter.IPAddress) if adapter.IPAddress else '无'
        mac_address = adapter.MACAddress if adapter.MACAddress else '无'
        status = '已启用' if adapter.IPEnabled else '未启用'
        description = adapter.Description
        adapters.append({
            '名称': description,
            'IP': ip_address,
            'MAC': mac_address,
            '状态': status
        })
    return adapters

def list_network_adapters_unix():
    result = subprocess.run(['ip', '-o', '-br', 'addr', 'show'], stdout=subprocess.PIPE)
    lines = result.stdout.decode().splitlines()
    adapters = []
    for line in lines:
        parts = line.split()
        name = parts[0]
        ips = [part.split('/')[0] for part in parts[2:] if part != 'inet' and part != 'inet6']
        ip_address = ', '.join(ips) if ips else '无'
        
        mac_result = subprocess.run(['cat', '/sys/class/net/{}/address'.format(name)], stdout=subprocess.PIPE)
        mac_address = mac_result.stdout.decode().strip() if mac_result.returncode == 0 else '无'
        
        status_result = subprocess.run(['cat', '/sys/class/net/{}/operstate'.format(name)], stdout=subprocess.PIPE)
        status = status_result.stdout.decode().strip() if status_result.returncode == 0 else '未知'
        
        adapters.append({
            '名称': name,
            'IP': ip_address,
            'MAC': mac_address,
            '状态': status
        })
    return adapters

def toggle_network_adapter(adapter_name):
    system_type = platform.system()
    if system_type == 'Windows':
        success, message = toggle_network_adapter_windows(adapter_name)
    elif system_type in ['Linux', 'Darwin']:
        success, message = toggle_network_adapter_unix(adapter_name)
    else:
        print("不支持的操作系统")
        return False, "不支持的操作系统"
    
    return success, message

# Windows-specific function to toggle network adapter state
def toggle_network_adapter_windows(adapter_name):
    try:
        c = wmi.WMI()
        adapters = c.Win32_NetworkAdapter(Description=adapter_name)
        if not adapters:
            print(f"找不到描述为 '{adapter_name}' 的网络适配器")
            return False, f"未找到名称为 {adapter_name} 的网卡"

        for adapter in adapters:
            current_status = adapter.NetConnectionStatus
            enable = current_status != 2  # 2 表示连接状态
            action = "启用" if enable else "禁用"
            
            try:
                action_result = adapter.Enable() if enable else adapter.Disable()
                if isinstance(action_result, tuple) and action_result[0] == 0:
                    print(f"{action}网卡 '{adapter.Description}' 成功")
                    return True, f"{adapter.Description} 已被成功{action}"
                else:
                    print(f"{action}网卡 '{adapter.Description}' 失败 - 返回值: {action_result}")
                    return False, f"{adapter.Description} {action}失败，返回值: {action_result}"
            except Exception as e:
                print(f"在尝试{action}网卡 '{adapter.Description}' 时发生错误: {e}")
                return False, f"{adapter.Description} {action}失败，错误: {str(e)}"
                
    except Exception as e:
        print(f"处理过程中发生了一个意外错误: {e}")
        return False, str(e)

# Linux/macOS-specific function to toggle network adapter state
def toggle_network_adapter_unix(adapter_name):
    try:
        check_cmd = f"ip link show {adapter_name}"
        if os.system(check_cmd) != 0:
            print(f"找不到名称为 '{adapter_name}' 的网络适配器")
            return False, f"未找到名称为 {adapter_name} 的网卡"

        status_cmd = f"cat /sys/class/net/{adapter_name}/operstate"
        current_state = os.popen(status_cmd).read().strip()
        enable = current_state != 'up'
        action = "启用" if enable else "禁用"

        print(f"当前网卡 '{adapter_name}' 状态为 {'已启用' if current_state == 'up' else '未启用'}")

        command = f"sudo ip link set dev {adapter_name} {'up' if enable else 'down'}"
        result_code = os.system(command)
        
        if result_code == 0:
            print(f"{action}网卡 '{adapter_name}' 成功")
            return True, f"{adapter_name} 已被成功{action}"
        else:
            print(f"{action}网卡 '{adapter_name}' 失败")
            return False, f"{adapter_name} {action}失败，错误代码: {result_code}"
    except Exception as e:
        print(f"处理过程中发生了一个意外错误: {e}")
        return False, str(e)

# 打印所有网络适配器的信息
def print_network_adapters_info(adapters):
    for idx, adapter in enumerate(adapters, 1):
        print(f"\n网卡 {idx}:")
        print(f"名称: {adapter['名称']}")
        print(f"IP 地址: {adapter['IP']}")
        print(f"MAC 地址: {adapter['MAC']}")
        print(f"状态: {adapter['状态']}")

# 示例调用
adapters = list_network_adapters()
print_network_adapters_info(adapters)

network_adapter_name = "Realtek PCIe GbE Family Controller"  # 修改为你的网卡名称
success, message = toggle_network_adapter(network_adapter_name)
print(f"\n操作结果: {success}, 详情: {message}")